@@ -49,7 +49,8 @@ LOG = logging.getLogger(__name__)
 _FILTERS_COLLECTION = None
 
 ATTRIBUTE_CONVERTERS = {'name~': 'display_name~',
-                        'description~': 'display_description~'}
+                        'description~': 'display_description~',
+                        'consumes_quota': 'use_quota'}
 
 
 METADATA_TYPES = enum.Enum('METADATA_TYPES', 'user image')
@@ -228,6 +229,14 @@ class ViewBuilder(object):
 
     _collection_name = None
 
+    def _get_project_id_in_url(self, request):
+        project_id = request.environ["cinder.context"].project_id
+        if project_id and ("/v3/%s" % project_id in request.url):
+            # project_ids are not mandatory within v3 URLs, but links need
+            # to include them if the request does.
+            return project_id
+        return ''
+
     def _get_links(self, request, identifier):
         return [{"rel": "self",
                  "href": self._get_href_link(request, identifier), },
@@ -241,7 +250,7 @@ class ViewBuilder(object):
         prefix = self._update_link_prefix(get_request_url(request),
                                           CONF.public_endpoint)
         url = os.path.join(prefix,
-                           request.environ["cinder.context"].project_id,
+                           self._get_project_id_in_url(request),
                            collection_name)
         return "%s?%s" % (url, urllib.parse.urlencode(params))
 
@@ -250,7 +259,7 @@ class ViewBuilder(object):
         prefix = self._update_link_prefix(get_request_url(request),
                                           CONF.public_endpoint)
         return os.path.join(prefix,
-                            request.environ["cinder.context"].project_id,
+                            self._get_project_id_in_url(request),
                             self._collection_name,
                             str(identifier))
 
@@ -260,7 +269,7 @@ class ViewBuilder(object):
         base_url = self._update_link_prefix(base_url,
                                             CONF.public_endpoint)
         return os.path.join(base_url,
-                            request.environ["cinder.context"].project_id,
+                            self._get_project_id_in_url(request),
                             self._collection_name,
                             str(identifier))
 
@@ -393,10 +402,10 @@ def get_enabled_resource_filters(resource=None):
         return {}
 
 
-def get_time_comparsion_operators():
-    """Get list of time comparsion operators.
+def get_time_comparison_operators():
+    """Get list of time comparison operators.
 
-    This method returns list which contains the allowed comparsion operators.
+    This method returns list which contains the allowed comparison operators.
     """
     return ["gt", "gte", "eq", "neq", "lt", "lte"]
 
