@@ -174,8 +174,8 @@ class CephBackupDriver(driver.BackupDriver):
     gain.
     """
 
-    def __init__(self, context, db=None, execute=None):
-        super(CephBackupDriver, self).__init__(context, db)
+    def __init__(self, context, execute=None):
+        super().__init__(context)
         self.rbd = rbd
         self.rados = rados
         self.chunk_size = CONF.backup_ceph_chunk_size
