@@ -13,7 +13,6 @@
 #    under the License.
 
 from oslo_serialization import jsonutils
-from oslo_utils import versionutils
 from oslo_versionedobjects import fields
 
 from cinder import db
@@ -37,6 +36,7 @@ class VolumeAttachment(base.CinderPersistentObject, base.CinderObject,
     OPTIONAL_FIELDS = ['volume']
     obj_extra_fields = ['project_id', 'volume_host']
 
+    # NOTE: When adding a field obj_make_compatible needs to be updated
     fields = {
         'id': fields.UUIDField(),
         'volume_id': fields.UUIDField(),
@@ -67,16 +67,6 @@ class VolumeAttachment(base.CinderPersistentObject, base.CinderObject,
     def _get_expected_attrs(cls, context, *args, **kwargs):
         return ['volume']
 
-    def obj_make_compatible(self, primitive, target_version):
-        """Make an object representation compatible with target version."""
-        super(VolumeAttachment, self).obj_make_compatible(primitive,
-                                                          target_version)
-        target_version = versionutils.convert_version_to_tuple(target_version)
-        if target_version < (1, 3):
-            primitive.pop('connector', None)
-        if target_version < (1, 2):
-            primitive.pop('connection_info', None)
-
     @classmethod
     def _from_db_object(cls, context, attachment, db_attachment,
                         expected_attrs=None):
