@@ -15,6 +15,7 @@
 
 import operator
 import re
+from typing import Callable, Dict  # noqa: H301
 
 import pyparsing
 
@@ -167,7 +168,7 @@ class EvalTernaryOp(object):
 
 
 class EvalFunction(object):
-    functions = {
+    functions: Dict[str, Callable] = {
         "abs": abs,
         "max": max,
         "min": min,
@@ -236,7 +237,7 @@ def _def_parser():
 
     oneOf = pyparsing.oneOf
     opAssoc = pyparsing.opAssoc
-    operatorPrecedence = pyparsing.operatorPrecedence
+    infixNotation = pyparsing.infixNotation
     Word = pyparsing.Word
 
     integer = Word(nums)
@@ -256,7 +257,7 @@ def _def_parser():
     negateop = oneOf('NOT not !')
 
     operand.setParseAction(EvalConstant)
-    expr = operatorPrecedence(operand, [
+    expr = infixNotation(operand, [
         (fn, 1, opAssoc.RIGHT, EvalFunction),
         ("^", 2, opAssoc.RIGHT, EvalPowerOp),
         (signop, 1, opAssoc.RIGHT, EvalSignOp),
