@@ -28,8 +28,6 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import timeutils
 
-from cinder.i18n import _LE
-
 
 scheduler_json_config_location_opt = cfg.StrOpt(
     'scheduler_json_config_location',
@@ -66,8 +64,8 @@ class SchedulerOptions(object):
         try:
             return os.path.getmtime(filename)
         except os.error:
-            LOG.exception(_LE("Could not stat scheduler options file "
-                              "%(filename)s."),
+            LOG.exception("Could not stat scheduler options file "
+                          "%(filename)s.",
                           {'filename': filename})
             raise
 
@@ -76,14 +74,14 @@ class SchedulerOptions(object):
         try:
             return json.load(handle)
         except ValueError:
-            LOG.exception(_LE("Could not decode scheduler options."))
+            LOG.exception("Could not decode scheduler options.")
             return {}
 
     def _get_time_now(self):
         """Get current UTC. Broken out for testing."""
         return timeutils.utcnow()
 
-    def get_configuration(self, filename=None):
+    def get_configuration(self, filename=None) -> dict:
         """Check the json file for changes and load it if needed."""
         if not filename:
             filename = CONF.scheduler_json_config_location
