@@ -37,8 +37,9 @@ class BaseHostWeigher(base_weight.BaseWeigher):
     pass
 
 
-class HostWeightHandler(base_weight.BaseWeightHandler):
+class OrderedHostWeightHandler(base_weight.BaseWeightHandler):
     object_class = WeighedHost
 
     def __init__(self, namespace):
-        super(HostWeightHandler, self).__init__(BaseHostWeigher, namespace)
+        super(OrderedHostWeightHandler, self).__init__(BaseHostWeigher,
+                                                       namespace)
