@@ -14,12 +14,12 @@
 #    under the License.
 
 import datetime
+from http import HTTPStatus
 from unittest import mock
 
 import ddt
 import iso8601
 from oslo_config import cfg
-from six.moves import http_client
 
 from cinder.api.contrib import services
 from cinder.api import extensions
@@ -637,7 +637,7 @@ class ServicesTest(test.TestCase):
     def test_services_enable_with_service_key(self):
         body = {'host': 'host1', 'service': constants.VOLUME_BINARY}
         req = fakes.HTTPRequest.blank(
-            '/v2/%s/os-services/enable' % fake.PROJECT_ID)
+            '/v3/%s/os-services/enable' % fake.PROJECT_ID)
         res_dict = self.controller.update(req, "enable", body)
 
         self.assertEqual('enabled', res_dict['status'])
@@ -645,14 +645,14 @@ class ServicesTest(test.TestCase):
     def test_services_enable_with_binary_key(self):
         body = {'host': 'host1', 'binary': constants.VOLUME_BINARY}
         req = fakes.HTTPRequest.blank(
-            '/v2/%s/os-services/enable' % fake.PROJECT_ID)
+            '/v3/%s/os-services/enable' % fake.PROJECT_ID)
         res_dict = self.controller.update(req, "enable", body)
 
         self.assertEqual('enabled', res_dict['status'])
 
     def test_services_disable_with_service_key(self):
         req = fakes.HTTPRequest.blank(
-            '/v2/%s/os-services/disable' % fake.PROJECT_ID)
+            '/v3/%s/os-services/disable' % fake.PROJECT_ID)
         body = {'host': 'host1', 'service': constants.VOLUME_BINARY}
         res_dict = self.controller.update(req, "disable", body)
 
@@ -660,7 +660,7 @@ class ServicesTest(test.TestCase):
 
     def test_services_disable_with_binary_key(self):
         req = fakes.HTTPRequest.blank(
-            '/v2/%s/os-services/disable' % fake.PROJECT_ID)
+            '/v3/%s/os-services/disable' % fake.PROJECT_ID)
         body = {'host': 'host1', 'binary': constants.VOLUME_BINARY}
         res_dict = self.controller.update(req, "disable", body)
 
@@ -687,7 +687,7 @@ class ServicesTest(test.TestCase):
             fakes.HTTPRequest.blank('v1/fake/os-services/disable-log-reason'))
         body = {'host': 'host1',
                 'binary': 'cinder-scheduler',
-                'disabled_reason': u'test-reason',
+                'disabled_reason': 'test-reason',
                 }
         res_dict = self.controller.update(req, "disable-log-reason", body)
 
@@ -723,7 +723,7 @@ class ServicesTest(test.TestCase):
                           req, "disable-log-reason", body)
 
     def test_services_failover_host(self):
-        url = '/v2/%s/os-services/failover_host' % fake.PROJECT_ID
+        url = '/v3/%s/os-services/failover_host' % fake.PROJECT_ID
         req = fakes.HTTPRequest.blank(url)
         body = {'host': 'fake_host',
                 'backend_id': 'fake_backend'}
@@ -734,7 +734,7 @@ class ServicesTest(test.TestCase):
                                               'fake_host',
                                               None,
                                               'fake_backend')
-        self.assertEqual(http_client.ACCEPTED, res.status_code)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_code)
 
     @ddt.data(('failover_host', {'host': 'fake_host',
                                  'backend_id': 'fake_backend'}),
@@ -744,7 +744,7 @@ class ServicesTest(test.TestCase):
     @mock.patch('cinder.objects.ServiceList.get_all')
     def test_services_action_host_not_found(self, method, body,
                                             mock_get_all_services):
-        url = '/v2/%s/os-services/%s' % (fake.PROJECT_ID, method)
+        url = '/v3/%s/os-services/%s' % (fake.PROJECT_ID, method)
         req = fakes.HTTPRequest.blank(url)
         mock_get_all_services.return_value = []
         msg = 'No service found with host=%s' % 'fake_host'
@@ -771,7 +771,7 @@ class ServicesTest(test.TestCase):
         self.assertEqual(msg, result.msg)
 
     def test_services_freeze(self):
-        url = '/v2/%s/os-services/freeze' % fake.PROJECT_ID
+        url = '/v3/%s/os-services/freeze' % fake.PROJECT_ID
         req = fakes.HTTPRequest.blank(url)
         body = {'host': 'fake_host'}
         with mock.patch.object(self.controller.volume_api, 'freeze_host') \
@@ -782,7 +782,7 @@ class ServicesTest(test.TestCase):
         self.assertEqual(freeze_mock.return_value, res)
 
     def test_services_thaw(self):
-        url = '/v2/%s/os-services/thaw' % fake.PROJECT_ID
+        url = '/v3/%s/os-services/thaw' % fake.PROJECT_ID
         req = fakes.HTTPRequest.blank(url)
         body = {'host': 'fake_host'}
         with mock.patch.object(self.controller.volume_api, 'thaw_host') \
@@ -794,7 +794,7 @@ class ServicesTest(test.TestCase):
 
     @ddt.data('freeze', 'thaw', 'failover_host')
     def test_services_replication_calls_no_host(self, method):
-        url = '/v2/%s/os-services/%s' % (fake.PROJECT_ID, method)
+        url = '/v3/%s/os-services/%s' % (fake.PROJECT_ID, method)
         req = fakes.HTTPRequest.blank(url)
         self.assertRaises(exception.InvalidInput,
                           self.controller.update, req, method, {})
@@ -824,32 +824,34 @@ class ServicesTest(test.TestCase):
                           self.controller._get_log, req, self.context,
                           body=body)
 
-    @mock.patch('cinder.api.contrib.services.ServiceController._get_log')
-    def test_get_log_w_server_filter(self, get_log_mock):
+    def test_get_log_w_server_filter_same_host(self):
         server_filter = 'controller-0'
-
-        body = {'host': CONF.host, 'server': server_filter}
+        CONF.set_override('host', server_filter)
+        body = {'binary': constants.API_BINARY, 'server': server_filter}
         req = FakeRequest(version=mv.LOG_LEVEL)
 
-        log_levels = self.controller._get_log(req, mock.sentinel.context, body)
+        log_levels = self.controller._get_log(
+            req=req, context=mock.sentinel.context, body=body)
+        log_levels = log_levels['log_levels']
 
-        self.assertEqual(get_log_mock.return_value, log_levels)
-        get_log_mock.assert_called_once_with(req, mock.sentinel.context, body)
-        self.assertNotEqual(log_levels['binary'], constants.API_BINARY)
-        self.assertIsNot(log_levels, constants.API_BINARY)
+        self.assertEqual(1, len(log_levels))
+        self.assertEqual('controller-0', log_levels[0]['host'])
+        self.assertEqual('cinder-api', log_levels[0]['binary'])
+        # since there are a lot of log levels, we just check if the key-value
+        # exists for levels
+        self.assertIsNotNone(log_levels[0]['levels'])
 
-    @mock.patch('cinder.api.contrib.services.ServiceController._get_log')
-    def test_get_log_w_server_equals_to_host(self, get_log_mock):
-        server_filter = 'server-0'
-
-        body = {'host': CONF.host, 'server': server_filter}
+    def test_get_log_w_server_filter_different_host(self):
+        server_filter = 'controller-0'
+        CONF.set_override('host', 'controller-different-host')
+        body = {'binary': constants.API_BINARY, 'server': server_filter}
         req = FakeRequest(version=mv.LOG_LEVEL)
 
-        log_levels = self.controller._get_log(req, mock.sentinel.context, body)
+        log_levels = self.controller._get_log(
+            req=req, context=mock.sentinel.context, body=body)
+        log_levels = log_levels['log_levels']
 
-        self.assertEqual(get_log_mock.return_value, log_levels)
-        self.assertNotEqual(log_levels['binary'], constants.API_BINARY)
-        self.assertIsNot(log_levels, constants.API_BINARY)
+        self.assertEqual(0, len(log_levels))
 
     @ddt.data(None, '', '*')
     @mock.patch('cinder.objects.ServiceList.get_all')
