@@ -13,12 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
+from http import HTTPStatus
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_serialization import jsonutils
-from six.moves import http_client
 import webob
 
 from cinder.api import extensions
@@ -27,11 +26,11 @@ from cinder.api.v2 import volumes
 from cinder import db
 from cinder import exception
 from cinder import objects
-from cinder import test
 from cinder.tests.unit.api import fakes
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder import volume
 from cinder.volume import api as volume_api
 
@@ -128,7 +127,7 @@ class VolumeMetaDataTest(test.TestCase):
         self.ext_mgr.extensions = {}
         self.volume_controller = volumes.VolumeController(self.ext_mgr)
         self.controller = volume_metadata.Controller()
-        self.req_id = str(uuid.uuid4())
+        self.req_id = fake.REQUEST_ID
         self.url = '/v2/%s/volumes/%s/metadata' % (
             fake.PROJECT_ID, self.req_id)
 
@@ -204,7 +203,7 @@ class VolumeMetaDataTest(test.TestCase):
                                'get') as get_volume:
             get_volume.return_value = fake_volume
             res = self.controller.delete(req, self.req_id, 'key2')
-            self.assertEqual(http_client.OK, res.status_int)
+            self.assertEqual(HTTPStatus.OK, res.status_int)
             get_volume.assert_called_once_with(fake_context, self.req_id)
 
     @mock.patch.object(db, 'volume_metadata_delete')
