@@ -27,8 +27,6 @@ from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from oslo_utils import units
-import six
-from six.moves import range
 
 from cinder.backup import driver
 from cinder.backup.drivers import ceph
@@ -237,11 +235,15 @@ class BackupCephTestCase(test.TestCase):
         del self.service.rbd.RBD_FEATURE_STRIPINGV2
         del self.service.rbd.RBD_FEATURE_EXCLUSIVE_LOCK
         del self.service.rbd.RBD_FEATURE_JOURNALING
+        del self.service.rbd.RBD_FEATURE_OBJECT_MAP
+        del self.service.rbd.RBD_FEATURE_FAST_DIFF
         self.assertFalse(hasattr(self.service.rbd, 'RBD_FEATURE_LAYERING'))
         self.assertFalse(hasattr(self.service.rbd, 'RBD_FEATURE_STRIPINGV2'))
         self.assertFalse(hasattr(self.service.rbd,
                                  'RBD_FEATURE_EXCLUSIVE_LOCK'))
         self.assertFalse(hasattr(self.service.rbd, 'RBD_FEATURE_JOURNALING'))
+        self.assertFalse(hasattr(self.service.rbd, 'RBD_FEATURE_OBJECT_MAP'))
+        self.assertFalse(hasattr(self.service.rbd, 'RBD_FEATURE_FAST_DIFF'))
 
         oldformat, features = self.service._get_rbd_support()
         self.assertTrue(oldformat)
@@ -281,6 +283,17 @@ class BackupCephTestCase(test.TestCase):
         self.assertFalse(oldformat)
         self.assertEqual(1 | 2 | 4 | 64, features)
 
+        #
+        # test that FAST_DIFF is enabled if supported by RBD
+        #   this also enables OBJECT_MAP as required by Ceph
+        #
+        self.service.rbd.RBD_FEATURE_OBJECT_MAP = 8
+        self.service.rbd.RBD_FEATURE_FAST_DIFF = 16
+
+        oldformat, features = self.service._get_rbd_support()
+        self.assertFalse(oldformat)
+        self.assertEqual(1 | 2 | 4 | 8 | 16 | 64, features)
+
     @common_mocks
     def test_get_backup_snap_name(self):
         snap_name = 'backup.%s.snap.3824923.1412' % (fake.VOLUME3_ID)
@@ -1342,7 +1355,7 @@ class BackupCephTestCase(test.TestCase):
             self.service._restore_metadata(self.backup, self.volume_id)
         except exception.BackupOperationError as exc:
             msg = _("Metadata restore failed due to incompatible version")
-            self.assertEqual(msg, six.text_type(exc))
+            self.assertEqual(msg, str(exc))
         else:
             # Force a test failure
             self.assertFalse(True)
@@ -1368,7 +1381,7 @@ class BackupCephTestCase(test.TestCase):
                 msg = (_("Failed to backup volume metadata - Metadata backup "
                          "object 'backup.%s.meta' already exists") %
                        (self.backup_id))
-                self.assertEqual(msg, six.text_type(e))
+                self.assertEqual(msg, str(e))
             else:
                 # Make the test fail
                 self.assertFalse(True)
