@@ -21,7 +21,6 @@ import zlib
 from googleapiclient import errors
 from oauth2client import client
 from oslo_utils import units
-import six
 
 
 class FakeGoogleObjectInsertExecute(object):
@@ -99,7 +98,7 @@ class FakeGoogleDiscovery(object):
         pass
 
     @classmethod
-    def Build(self, *args, **kargs):
+    def Build(cls, *args, **kargs):
         return FakeDiscoveryBuild()
 
 
@@ -120,7 +119,7 @@ class FakeGoogleCredentials(object):
         pass
 
     @classmethod
-    def from_stream(self, *args, **kwargs):
+    def from_stream(cls, *args, **kwargs):
         pass
 
 
@@ -144,8 +143,7 @@ class FakeGoogleMediaIoBaseDownload(object):
                                'offset': 20}
             }]
             metadata_json = json.dumps(metadata, sort_keys=True, indent=2)
-            if six.PY3:
-                metadata_json = metadata_json.encode('utf-8')
+            metadata_json = metadata_json.encode('utf-8')
             fh.write(metadata_json)
         else:
             fh.write(zlib.compress(os.urandom(units.Mi)))
