@@ -16,10 +16,13 @@
 Unit Tests for cinder.backup.rpcapi
 """
 
+from unittest import mock
+
 from cinder.backup import rpcapi as backup_rpcapi
-from cinder import test
+from cinder import objects
 from cinder.tests.unit.backup import fake_backup
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 
 
 class BackupRPCAPITestCase(test.RPCAPITestCase):
@@ -38,7 +41,7 @@ class BackupRPCAPITestCase(test.RPCAPITestCase):
         self._test_rpc_api('restore_backup',
                            rpc_method='cast',
                            server='fake_volume_host',
-                           volume_host='fake_volume_host',
+                           backup_host='fake_volume_host',
                            backup=self.fake_backup_obj,
                            volume_id=fake.VOLUME_ID)
 
@@ -79,3 +82,23 @@ class BackupRPCAPITestCase(test.RPCAPITestCase):
                            server='fake_volume_host',
                            host='fake_volume_host',
                            retval=True)
+
+    @mock.patch('oslo_messaging.RPCClient.can_send_version', mock.Mock())
+    def test_set_log_levels(self):
+        service = objects.Service(self.context, host='host1')
+        self._test_rpc_api('set_log_levels',
+                           rpc_method='cast',
+                           server=service.host,
+                           service=service,
+                           log_request='log_request',
+                           version='2.1')
+
+    @mock.patch('oslo_messaging.RPCClient.can_send_version', mock.Mock())
+    def test_get_log_levels(self):
+        service = objects.Service(self.context, host='host1')
+        self._test_rpc_api('get_log_levels',
+                           rpc_method='call',
+                           server=service.host,
+                           service=service,
+                           log_request='log_request',
+                           version='2.1')
