@@ -11,33 +11,219 @@
 #    under the License.
 
 """
-Tests for database migrations. This test case reads the configuration
-file test_migrations.conf for database connection settings
-to use in the tests. For each connection found in the config file,
+Tests for database migrations. For each database backend supported by cinder,
 the test case runs a series of test cases to ensure that migrations work
-properly both upgrading and downgrading, and that no data loss occurs
-if possible.
+properly and that no data loss occurs if possible.
 """
 
 import os
 
+from alembic import command as alembic_api
+from alembic import script as alembic_script
 import fixtures
-from migrate.versioning import api as migration_api
+from migrate.versioning import api as migrate_api
 from migrate.versioning import repository
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import test_fixtures
 from oslo_db.sqlalchemy import test_migrations
 from oslo_db.sqlalchemy import utils as db_utils
+from oslo_log.fixture import logging_error as log_fixture
 from oslotest import base as test_base
 import sqlalchemy
 from sqlalchemy.engine import reflection
 
+import cinder.db.legacy_migrations
 from cinder.db import migration
-import cinder.db.sqlalchemy.migrate_repo
+from cinder.db.sqlalchemy import models
+from cinder.tests import fixtures as cinder_fixtures
 from cinder.tests.unit import utils as test_utils
+from cinder.volume import volume_types
 
 
-class MigrationsMixin(test_migrations.WalkVersionsMixin):
+class CinderModelsMigrationsSync(test_migrations.ModelsMigrationsSync):
+    """Test sqlalchemy-migrate migrations."""
+
+    # Migrations can take a long time, particularly on underpowered CI nodes.
+    # Give them some breathing room.
+    TIMEOUT_SCALING_FACTOR = 4
+
+    def setUp(self):
+        # Ensure BaseTestCase's ConfigureLogging fixture is disabled since
+        # we're using our own (StandardLogging).
+        with fixtures.EnvironmentVariable('OS_LOG_CAPTURE', '0'):
+            super().setUp()
+
+        self.useFixture(log_fixture.get_logging_handle_error_fixture())
+        self.useFixture(cinder_fixtures.WarningsFixture())
+        self.useFixture(cinder_fixtures.StandardLogging())
+
+        self.engine = enginefacade.writer.get_engine()
+
+    def db_sync(self, engine):
+        migration.db_sync(engine=self.engine)
+
+    def get_engine(self):
+        return self.engine
+
+    def get_metadata(self):
+        return models.BASE.metadata
+
+    def include_object(self, object_, name, type_, reflected, compare_to):
+        if type_ == 'table':
+            # migrate_version is a sqlalchemy-migrate control table and
+            # isn't included in the model
+            if name == 'migrate_version':
+                return False
+
+        return True
+
+    def filter_metadata_diff(self, diff):
+        # Overriding the parent method to decide on certain attributes
+        # that maybe present in the DB but not in the models.py
+
+        def include_element(element):
+            """Determine whether diff element should be excluded."""
+            if element[0] == 'modify_nullable':
+                table_name, column = element[2], element[3]
+                return (table_name, column) not in {
+                    # NOTE(stephenfin): This field has nullable=True set, but
+                    # since it's also a primary key (primary_key=True) the
+                    # resulting schema will still end up being "NOT NULL". This
+                    # weird combination was deemed necessary because MySQL will
+                    # otherwise set this to "NOT NULL DEFAULT ''" which may be
+                    # harmless but is inconsistent with other models. See the
+                    # migration for more information.
+                    ('encryption', 'encryption_id'),
+                    # NOTE(stephenfin): The nullability of these fields is
+                    # dependent on the backend, for some reason
+                    ('encryption', 'provider'),
+                    ('encryption', 'control_location'),
+                }
+
+            return True
+
+        return [element for element in diff if include_element(element[0])]
+
+
+class TestModelsSyncSQLite(
+    CinderModelsMigrationsSync,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    pass
+
+
+class TestModelsSyncMySQL(
+    CinderModelsMigrationsSync,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
+
+
+class TestModelsSyncPostgreSQL(
+    CinderModelsMigrationsSync,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
+
+
+class MigrationsWalk(
+    test_fixtures.OpportunisticDBTestMixin, test_base.BaseTestCase,
+):
+    # Migrations can take a long time, particularly on underpowered CI nodes.
+    # Give them some breathing room.
+    TIMEOUT_SCALING_FACTOR = 4
+
+    def setUp(self):
+        super().setUp()
+        self.engine = enginefacade.writer.get_engine()
+        self.config = migration._find_alembic_conf()
+        self.init_version = migration.ALEMBIC_INIT_VERSION
+
+    def _migrate_up(self, revision, connection):
+        check_method = getattr(self, f'_check_{revision}', None)
+        if revision != self.init_version:  # no tests for the initial revision
+            self.assertIsNotNone(
+                check_method,
+                f"API DB Migration {revision} doesn't have a test; add one"
+            )
+
+        pre_upgrade = getattr(self, f'_pre_upgrade_{revision}', None)
+        if pre_upgrade:
+            pre_upgrade(connection)
+
+        alembic_api.upgrade(self.config, revision)
+
+        if check_method:
+            check_method(connection)
+
+    def test_single_base_revision(self):
+        """Ensure we only have a single base revision.
+
+        There's no good reason for us to have diverging history, so validate
+        that only one base revision exists. This will prevent simple errors
+        where people forget to specify the base revision. If this fail for your
+        change, look for migrations that do not have a 'revises' line in them.
+        """
+        script = alembic_script.ScriptDirectory.from_config(self.config)
+        self.assertEqual(1, len(script.get_bases()))
+
+    def test_single_head_revision(self):
+        """Ensure we only have a single head revision.
+
+        There's no good reason for us to have diverging history, so validate
+        that only one head revision exists. This will prevent merge conflicts
+        adding additional head revision points. If this fail for your change,
+        look for migrations with the same 'revises' line in them.
+        """
+        script = alembic_script.ScriptDirectory.from_config(self.config)
+        self.assertEqual(1, len(script.get_heads()))
+
+    def test_walk_versions(self):
+        with self.engine.begin() as connection:
+            self.config.attributes['connection'] = connection
+            script = alembic_script.ScriptDirectory.from_config(self.config)
+            revisions = list(script.walk_revisions())
+            # Need revisions from older to newer so the walk works as intended
+            revisions.reverse()
+            for revision_script in revisions:
+                self._migrate_up(revision_script.revision, connection)
+
+    def test_db_version_alembic(self):
+        migration.db_sync()
+        head = alembic_script.ScriptDirectory.from_config(
+            self.config,
+        ).get_current_head()
+        self.assertEqual(head, migration.db_version())
+
+
+class TestMigrationsWalkSQLite(
+    MigrationsWalk,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    pass
+
+
+class TestMigrationsWalkMySQL(
+    MigrationsWalk,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
+
+
+class TestMigrationsWalkPostgreSQL(
+    MigrationsWalk,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
+
+
+class LegacyMigrationsWalk(test_migrations.WalkVersionsMixin):
     """Test sqlalchemy-migrate migrations."""
 
     BOOL_TYPE = sqlalchemy.types.BOOLEAN
@@ -46,28 +232,23 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
     VARCHAR_TYPE = sqlalchemy.types.VARCHAR
     TEXT_TYPE = sqlalchemy.types.Text
 
+    def setUp(self):
+        super().setUp()
+        self.engine = enginefacade.writer.get_engine()
+
     @property
     def INIT_VERSION(self):
-        return migration.INIT_VERSION
+        return migration.MIGRATE_INIT_VERSION
 
     @property
     def REPOSITORY(self):
-        migrate_file = cinder.db.sqlalchemy.migrate_repo.__file__
+        migrate_file = cinder.db.legacy_migrations.__file__
         return repository.Repository(
             os.path.abspath(os.path.dirname(migrate_file)))
 
     @property
     def migration_api(self):
-        return migration_api
-
-    def setUp(self):
-        super(MigrationsMixin, self).setUp()
-
-        # (zzzeek) This mixin states that it uses the
-        # "self.engine" attribute in the migrate_engine() method.
-        # So the mixin must set that up for itself, oslo_db no longer
-        # makes these assumptions for you.
-        self.engine = enginefacade.writer.get_engine()
+        return migrate_api
 
     @property
     def migrate_engine(self):
@@ -80,7 +261,7 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
     class BannedDBSchemaOperations(fixtures.Fixture):
         """Ban some operations for migrations"""
         def __init__(self, banned_resources=None):
-            super(MigrationsMixin.BannedDBSchemaOperations, self).__init__()
+            super().__init__()
             self._banned_resources = banned_resources or []
 
         @staticmethod
@@ -91,7 +272,7 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
                     resource, op))
 
         def setUp(self):
-            super(MigrationsMixin.BannedDBSchemaOperations, self).setUp()
+            super().setUp()
             for thing in self._banned_resources:
                 self.useFixture(fixtures.MonkeyPatch(
                     'sqlalchemy.%s.drop' % thing,
@@ -107,23 +288,24 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
         # manner is provided in Cinder's developer documentation.
         # Reviewers: DO NOT ALLOW THINGS TO BE ADDED HERE WITHOUT CARE
         exceptions = [
-            # NOTE : 104 modifies size of messages.project_id to 255.
-            # This should be safe according to documentation.
-            104,
-            # NOTE(brinzhang): 127 changes size of quota_usage.resource
+            # NOTE(brinzhang): 135 changes size of quota_usage.resource
             # to 300. This should be safe for the 'quota_usage' db table,
             # because of the 255 is the length limit of volume_type_name,
             # it should be add the additional prefix before volume_type_name,
             # which we of course allow *this* size to 300.
-            127,
+            135,
+            # 136 modifies the the tables having volume_type_id field to set
+            # as non nullable
+            136,
         ]
 
         if version not in exceptions:
             banned = ['Table', 'Column']
         else:
             banned = None
-        with MigrationsMixin.BannedDBSchemaOperations(banned):
-            super(MigrationsMixin, self).migrate_up(version, with_data)
+
+        with LegacyMigrationsWalk.BannedDBSchemaOperations(banned):
+            super().migrate_up(version, with_data)
 
     def __check_cinderbase_fields(self, columns):
         """Check fields inherited from CinderBase ORM class."""
@@ -132,20 +314,6 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
         self.assertIsInstance(columns.deleted_at.type, self.TIME_TYPE)
         self.assertIsInstance(columns.deleted.type, self.BOOL_TYPE)
 
-    def _check_098(self, engine, data):
-        self.assertTrue(engine.dialect.has_table(engine.connect(),
-                                                 "messages"))
-        ids = self.get_indexed_columns(engine, 'messages')
-        self.assertTrue('expires_at' in ids)
-
-    def _check_099(self, engine, data):
-        self.assertTrue(engine.dialect.has_table(engine.connect(),
-                                                 "volume_attachment"))
-        attachment = db_utils.get_table(engine, 'volume_attachment')
-
-        self.assertIsInstance(attachment.c.connection_info.type,
-                              self.TEXT_TYPE)
-
     def get_table_names(self, engine):
         inspector = reflection.Inspector.from_engine(engine)
         return inspector.get_table_names()
@@ -183,159 +351,6 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
 
         self.assertSetEqual(set(), non_indexed_foreign_keys)
 
-    def _pre_upgrade_101(self, engine):
-        """Add data to test the SQL migration."""
-
-        types_table = db_utils.get_table(engine, 'volume_types')
-        for i in range(1, 5):
-            types_table.insert().execute({'id': str(i)})
-
-        specs_table = db_utils.get_table(engine, 'volume_type_extra_specs')
-        specs = [
-            {'volume_type_id': '1', 'key': 'key', 'value': '<is> False'},
-            {'volume_type_id': '2', 'key': 'replication_enabled',
-             'value': '<is> False'},
-            {'volume_type_id': '3', 'key': 'replication_enabled',
-             'value': '<is> True', 'deleted': True},
-            {'volume_type_id': '3', 'key': 'key', 'value': '<is> True'},
-            {'volume_type_id': '4', 'key': 'replication_enabled',
-             'value': '<is> True'},
-            {'volume_type_id': '4', 'key': 'key', 'value': '<is> True'},
-        ]
-        for spec in specs:
-            specs_table.insert().execute(spec)
-
-        volumes_table = db_utils.get_table(engine, 'volumes')
-        volumes = [
-            {'id': '1', 'replication_status': 'disabled',
-             'volume_type_id': None},
-            {'id': '2', 'replication_status': 'disabled',
-             'volume_type_id': ''},
-            {'id': '3', 'replication_status': 'disabled',
-             'volume_type_id': '1'},
-            {'id': '4', 'replication_status': 'disabled',
-             'volume_type_id': '2'},
-            {'id': '5', 'replication_status': 'disabled',
-             'volume_type_id': '2'},
-            {'id': '6', 'replication_status': 'disabled',
-             'volume_type_id': '3'},
-            {'id': '7', 'replication_status': 'error', 'volume_type_id': '4'},
-            {'id': '8', 'deleted': True, 'replication_status': 'disabled',
-             'volume_type_id': '4'},
-            {'id': '9', 'replication_status': 'disabled', 'deleted': None,
-             'volume_type_id': '4'},
-            {'id': '10', 'replication_status': 'disabled', 'deleted': False,
-             'volume_type_id': '4'},
-        ]
-        for volume in volumes:
-            volumes_table.insert().execute(volume)
-
-        # Only the last volume should be changed to enabled
-        expected = {v['id']: v['replication_status'] for v in volumes}
-        expected['9'] = 'enabled'
-        expected['10'] = 'enabled'
-        return expected
-
-    def _check_101(self, engine, data):
-        # Get existing volumes after the migration
-        volumes_table = db_utils.get_table(engine, 'volumes')
-        volumes = volumes_table.select().execute()
-        # Check that the replication_status is the one we expect according to
-        # _pre_upgrade_098
-        for volume in volumes:
-            self.assertEqual(data[volume.id], volume.replication_status,
-                             'id %s' % volume.id)
-
-    def _check_102(self, engine, data):
-        """Test adding replication_status to groups table."""
-        groups = db_utils.get_table(engine, 'groups')
-        self.assertIsInstance(groups.c.replication_status.type,
-                              self.VARCHAR_TYPE)
-
-    def _check_103(self, engine, data):
-        self.assertTrue(engine.dialect.has_table(engine.connect(),
-                                                 "messages"))
-        attachment = db_utils.get_table(engine, 'messages')
-
-        self.assertIsInstance(attachment.c.detail_id.type,
-                              self.VARCHAR_TYPE)
-        self.assertIsInstance(attachment.c.action_id.type,
-                              self.VARCHAR_TYPE)
-
-    def _check_104(self, engine, data):
-        messages = db_utils.get_table(engine, 'messages')
-        self.assertEqual(255, messages.c.project_id.type.length)
-
-    def _check_105(self, engine, data):
-        self.assertTrue(engine.dialect.has_table(engine.connect(),
-                                                 "backup_metadata"))
-        backup_metadata = db_utils.get_table(engine, 'backup_metadata')
-
-        self.assertIsInstance(backup_metadata.c.created_at.type,
-                              self.TIME_TYPE)
-        self.assertIsInstance(backup_metadata.c.updated_at.type,
-                              self.TIME_TYPE)
-        self.assertIsInstance(backup_metadata.c.deleted_at.type,
-                              self.TIME_TYPE)
-        self.assertIsInstance(backup_metadata.c.deleted.type,
-                              self.BOOL_TYPE)
-        self.assertIsInstance(backup_metadata.c.id.type,
-                              self.INTEGER_TYPE)
-        self.assertIsInstance(backup_metadata.c.key.type,
-                              self.VARCHAR_TYPE)
-        self.assertIsInstance(backup_metadata.c.value.type,
-                              self.VARCHAR_TYPE)
-        self.assertIsInstance(backup_metadata.c.backup_id.type,
-                              self.VARCHAR_TYPE)
-        f_keys = self.get_foreign_key_columns(engine, 'backup_metadata')
-        self.assertEqual({'backup_id'}, f_keys)
-
-    def _check_111(self, engine, data):
-        self.assertTrue(db_utils.index_exists_on_columns(
-            engine, 'quota_usages', ['project_id', 'resource']))
-
-    def _check_112(self, engine, data):
-        services = db_utils.get_table(engine, 'services')
-        self.assertIsInstance(services.c.uuid.type,
-                              self.VARCHAR_TYPE)
-
-    def _check_113(self, engine, data):
-        """Test that adding reservations index works correctly."""
-        reservations = db_utils.get_table(engine, 'reservations')
-        index_columns = []
-        for idx in reservations.indexes:
-            if idx.name == 'reservations_deleted_uuid_idx':
-                index_columns = idx.columns.keys()
-                break
-
-        self.assertEqual(sorted(['deleted', 'uuid']),
-                         sorted(index_columns))
-
-    def _check_114(self, engine, data):
-        volumes = db_utils.get_table(engine, 'volumes')
-        self.assertIsInstance(volumes.c.service_uuid.type,
-                              self.VARCHAR_TYPE)
-        index_columns = []
-        for idx in volumes.indexes:
-            if idx.name == 'volumes_service_uuid_idx':
-                index_columns = idx.columns.keys()
-                break
-        self.assertEqual(sorted(['deleted', 'service_uuid']),
-                         sorted(index_columns))
-
-    def _check_115(self, engine, data):
-        volumes = db_utils.get_table(engine, 'volumes')
-        self.assertIsInstance(volumes.c.shared_targets.type,
-                              self.BOOL_TYPE)
-
-    def _check_116(self, engine, data):
-        volume_attachment = db_utils.get_table(engine, 'volume_attachment')
-        self.assertIn('connector', volume_attachment.c)
-
-    def _check_123(self, engine, data):
-        volume_transfer = db_utils.get_table(engine, 'transfers')
-        self.assertIn('no_snapshots', volume_transfer.c)
-
     def _check_127(self, engine, data):
         quota_usage_resource = db_utils.get_table(engine, 'quota_usages')
         self.assertIn('resource', quota_usage_resource.c)
@@ -349,6 +364,31 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
         self.assertIn('destination_project_id', volume_transfer.c)
         self.assertIn('accepted', volume_transfer.c)
 
+    def _check_132(self, engine, data):
+        """Test create default volume type."""
+        vol_types = db_utils.get_table(engine, 'volume_types')
+        vtype = (vol_types.select(vol_types.c.name ==
+                                  volume_types.DEFAULT_VOLUME_TYPE)
+                 .execute().first())
+        self.assertIsNotNone(vtype)
+
+    def _check_136(self, engine, data):
+        """Test alter volume_type_id columns."""
+        vol_table = db_utils.get_table(engine, 'volumes')
+        snap_table = db_utils.get_table(engine, 'snapshots')
+        encrypt_table = db_utils.get_table(engine, 'encryption')
+        self.assertFalse(vol_table.c.volume_type_id.nullable)
+        self.assertFalse(snap_table.c.volume_type_id.nullable)
+        self.assertFalse(encrypt_table.c.volume_type_id.nullable)
+
+    def _check_145(self, engine, data):
+        """Test add use_quota columns."""
+        for name in ('volumes', 'snapshots'):
+            resources = db_utils.get_table(engine, name)
+            self.assertIn('use_quota', resources.c)
+            # TODO: (Y release) Alter in new migration & change to assertFalse
+            self.assertTrue(resources.c.use_quota.nullable)
+
     # NOTE: this test becomes slower with each addition of new DB migration.
     # 'pymysql' works much slower on slow nodes than 'psycopg2'. And such
     # timeout mostly required for testing of 'mysql' backend.
@@ -358,9 +398,11 @@ class MigrationsMixin(test_migrations.WalkVersionsMixin):
         self.assert_each_foreign_key_is_part_of_an_index()
 
 
-class TestSqliteMigrations(test_fixtures.OpportunisticDBTestMixin,
-                           MigrationsMixin,
-                           test_base.BaseTestCase):
+class TestLegacyMigrationsWalkSQLite(
+    test_fixtures.OpportunisticDBTestMixin,
+    LegacyMigrationsWalk,
+    test_base.BaseTestCase,
+):
 
     def assert_each_foreign_key_is_part_of_an_index(self):
         # Skip the test for SQLite because SQLite does not list
@@ -369,9 +411,11 @@ class TestSqliteMigrations(test_fixtures.OpportunisticDBTestMixin,
         pass
 
 
-class TestMysqlMigrations(test_fixtures.OpportunisticDBTestMixin,
-                          MigrationsMixin,
-                          test_base.BaseTestCase):
+class TestLegacyMigrationsWalkMySQL(
+    test_fixtures.OpportunisticDBTestMixin,
+    LegacyMigrationsWalk,
+    test_base.BaseTestCase,
+):
 
     FIXTURE = test_fixtures.MySQLOpportunisticFixture
     BOOL_TYPE = sqlalchemy.dialects.mysql.TINYINT
@@ -382,7 +426,10 @@ class TestMysqlMigrations(test_fixtures.OpportunisticDBTestMixin,
         # add this to the global lists to make reset work with it, it's removed
         # automatically in tearDown so no need to clean it up here.
         # sanity check
-        migration.db_sync(engine=self.migrate_engine)
+        repo = migration._find_migrate_repo()
+        migrate_api.version_control(
+            self.migrate_engine, repo, migration.MIGRATE_INIT_VERSION)
+        migrate_api.upgrade(self.migrate_engine, repo)
 
         total = self.migrate_engine.execute(
             "SELECT count(*) "
@@ -409,13 +456,14 @@ class TestMysqlMigrations(test_fixtures.OpportunisticDBTestMixin,
         # Depending on the MariaDB version, and the page size, we may not have
         # been able to change quota_usage_resource to 300 chars, it could still
         # be 255.
-        self.assertIn(quota_usage_resource.c.resource.type.length,
-                      (255, 300))
+        self.assertIn(quota_usage_resource.c.resource.type.length, (255, 300))
 
 
-class TestPostgresqlMigrations(test_fixtures.OpportunisticDBTestMixin,
-                               MigrationsMixin,
-                               test_base.BaseTestCase):
+class TestLegacyMigrationsWalkPostgreSQL(
+    test_fixtures.OpportunisticDBTestMixin,
+    LegacyMigrationsWalk,
+    test_base.BaseTestCase,
+):
 
     FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
     TIME_TYPE = sqlalchemy.types.TIMESTAMP
