@@ -13,17 +13,17 @@
 #    under the License.
 
 from datetime import timedelta
-import ddt
-import mock
+from unittest import mock
 
+import ddt
 from oslo_utils import timeutils
 
 from cinder import context as ctxt
 from cinder.db.sqlalchemy import models
 from cinder.image import cache as image_cache
 from cinder import objects
-from cinder import test
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 
 
 @ddt.ddt
