@@ -16,6 +16,7 @@
 
 import datetime
 import itertools
+import traceback
 from unittest import mock
 
 import ddt
@@ -29,7 +30,7 @@ from cinder import context
 from cinder import exception
 from cinder.image import glance
 from cinder import service_auth
-from cinder.tests.unit.glance import stubs as glance_stubs
+from cinder.tests.unit.image import glance_stubs
 from cinder.tests.unit import test
 
 
@@ -251,6 +252,36 @@ class TestGlanceImageService(test.TestCase):
         actual = self.service.show(self.context, image_id)
         self.assertDictEqual(expected, actual)
 
+    def test_create_without_is_public(self):
+        """Test Creating images without is_public.
+
+        Ensure we can create an image without is_public attribute.
+        """
+        fixture = {'name': 'test image',
+                   'protected': False}
+        image_id = self.service.create(self.context, fixture)['id']
+
+        expected = {
+            'id': image_id,
+            'name': 'test image',
+            'protected': False,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {},
+            'owner': None,
+            'visibility': None,
+        }
+        actual = self.service.show(self.context, image_id)
+        self.assertDictEqual(expected, actual)
+
     def test_show_shared_image_membership_success(self):
         """Test Create Shared Image Membership Success
 
@@ -613,6 +644,61 @@ class TestGlanceImageService(test.TestCase):
         }
         self.assertEqual(expected, image_meta)
 
+    def test_show_passes_when_is_admin_in_the_context(self):
+        fixture = self._make_fixture(name='image2')
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = True
+
+        image_meta = self.service.show(self.context, image_id)
+        expected = {
+            'id': image_id,
+            'name': 'image2',
+            'protected': None,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'properties': {}},
+            'owner': None,
+            'visibility': None
+        }
+        self.assertEqual(expected, image_meta)
+
+    def test_show_passes_when_is_public_in_visibility_param(self):
+        fixture = self._make_fixture(name='image3')
+        fixture['visibility'] = 'public'
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = False
+
+        image_meta = self.service.show(self.context, image_id)
+        expected = {
+            'id': image_id,
+            'name': 'image3',
+            'protected': None,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'properties': {}},
+            'owner': None,
+            'visibility': 'public'
+        }
+        self.assertEqual(expected, image_meta)
+
     def test_show_raises_when_no_authtoken_in_the_context(self):
         fixture = self._make_fixture(name='image1',
                                      is_public=False,
@@ -624,6 +710,30 @@ class TestGlanceImageService(test.TestCase):
                           self.context,
                           image_id)
 
+    def test_show_raises_when_no_is_admin_in_the_context(self):
+        fixture = self._make_fixture(name='image2',
+                                     is_public=False,
+                                     protected=False)
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = False
+        self.assertRaises(exception.ImageNotFound,
+                          self.service.show,
+                          self.context,
+                          image_id)
+
+    def test_show_raises_when_is_private_in_visibility_param(self):
+        fixture = self._make_fixture(name='image3',
+                                     protected=False)
+        fixture['visibility'] = 'private'
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = False
+        self.assertRaises(exception.ImageNotFound,
+                          self.service.show,
+                          self.context,
+                          image_id)
+
     def test_detail_passes_through_to_client(self):
         fixture = self._make_fixture(name='image10', is_public=True)
         image_id = self.service.create(self.context, fixture)['id']
@@ -721,6 +831,62 @@ class TestGlanceImageService(test.TestCase):
         self.assertRaises(exception.ImageNotAuthorized, service.download,
                           self.context, image_id, writer)
 
+    def test_client_translated_exc_includes_original_traceback(self):
+        image_id = 1  # doesn't matter
+        original_exc = glanceclient.exc.Forbidden(image_id)
+
+        class MyGlanceStubClient(glance_stubs.StubGlanceClient):
+            def get(self, image_id):
+                raise original_exc
+
+        client = MyGlanceStubClient()
+        service = self._create_image_service(client)
+        writer = NullWriter()
+
+        exc = self.assertRaises(exception.ImageNotAuthorized,
+                                service.download,
+                                self.context,
+                                image_id,
+                                writer)
+
+        original = traceback.extract_tb(original_exc.__traceback__)
+        original.reverse()
+        received = traceback.extract_tb(exc.__traceback__)
+        received.reverse()
+        # verify that we have the same traceback as original_exc
+        for orig, recd in zip(original, received):
+            self.assertEqual(orig, recd)
+
+        # note that the received exception contains more frames in
+        # its traceback than the original
+        self.assertGreater(len(received), len(original))
+
+    def test_plain_translated_exc_includes_original_traceback(self):
+        original_exc = glanceclient.exc.Forbidden()
+
+        class MyGlanceStubClient(glance_stubs.StubGlanceClient):
+            def list(self):
+                raise original_exc
+
+        client = MyGlanceStubClient()
+        service = self._create_image_service(client)
+
+        exc = self.assertRaises(exception.NotAuthorized,
+                                service.detail,
+                                self.context)
+
+        original = traceback.extract_tb(original_exc.__traceback__)
+        original.reverse()
+        received = traceback.extract_tb(exc.__traceback__)
+        received.reverse()
+        # verify that we have the same traceback as original_exc
+        for orig, recd in zip(original, received):
+            self.assertEqual(orig, recd)
+
+        # note that the received exception contains more frames in
+        # its traceback than the original
+        self.assertGreater(len(received), len(original))
+
     def test_client_httpforbidden_converts_to_imagenotauthed(self):
         class MyGlanceStubClient(glance_stubs.StubGlanceClient):
             """A client that raises a HTTPForbidden exception."""
@@ -773,6 +939,7 @@ class TestGlanceImageService(test.TestCase):
         self.flags(allowed_direct_url_schemes=['file'])
         self.service.download(self.context, image_id, writer)
         mock_copyfileobj.assert_called_once_with(mock.ANY, writer)
+        mock_open.assert_called_once_with('/tmp/test', 'rb')
 
     @mock.patch('six.moves.builtins.open')
     @mock.patch('shutil.copyfileobj')
@@ -1041,8 +1208,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(auth_strategy='keystone')
         self.flags(glance_request_timeout=None)
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual("http://fake_host:9292", args[0])
                 self.assertNotIn('timeout', kwargs)
@@ -1051,6 +1218,8 @@ class TestGlanceImageServiceClient(test.TestCase):
 
         config_options = {'insecure': False,
                           'cacert': None,
+                          'key': None,
+                          'cert': None,
                           'timeout': None,
                           'split_loggers': False}
 
@@ -1072,9 +1241,11 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(glance_request_timeout=60)
         self.flags(
             glance_ca_certificates_file='/opt/stack/data/ca-bundle.pem')
+        self.flags(glance_certfile='/opt/stack/data/cert.pem')
+        self.flags(glance_keyfile='/opt/stack/data/key.pem')
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual("https://fake_host:9292", args[0])
                 self.assertNotIn('timeout', kwargs)
@@ -1083,6 +1254,8 @@ class TestGlanceImageServiceClient(test.TestCase):
 
         config_options = {'insecure': False,
                           'cacert': '/opt/stack/data/ca-bundle.pem',
+                          'cert': '/opt/stack/data/cert.pem',
+                          'key': '/opt/stack/data/key.pem',
                           'timeout': 60,
                           'split_loggers': False}
 
@@ -1103,8 +1276,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(
             glance_ca_certificates_file='/opt/stack/data/ca-bundle.pem')
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual('https://fake_host:9292', args[0])
                 self.assertEqual(60, kwargs['timeout'])
@@ -1124,8 +1297,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(auth_strategy='noauth')
         self.flags(glance_request_timeout=None)
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual("http://fake_host:9292", args[0])
                 self.assertNotIn('timeout', kwargs)
