@@ -10,7 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
 from cinder.tests.unit.targets import targets_fixture as tf
 from cinder import utils
