@@ -13,14 +13,14 @@
 import os
 import sys
 import time
+from unittest import mock
 
-import mock
 from oslo_concurrency import processutils as putils
 
 from cinder import context
 from cinder import exception
-from cinder import test
 from cinder.tests.unit.targets import targets_fixture as tf
+from cinder.tests.unit import test
 from cinder import utils
 from cinder.volume.targets import tgt
 from cinder.volume import volume_utils
