@@ -229,8 +229,8 @@ class NotifyUsageTestCase(test.TestCase):
             'user_id': '158cba1b8c2bb6008e',
             'host': 'fake_host',
             'availability_zone': 'nova',
-            'volume_type_id': 'fake_volume_type_id',
-            'id': 'fake_volume_id',
+            'volume_type_id': fake.VOLUME_TYPE_ID,
+            'id': fake.VOLUME_ID,
             'size': 1,
             'display_name': 'test_volume',
             'created_at': datetime.datetime(2015, 1, 1, 1, 1, 1),
@@ -250,8 +250,8 @@ class NotifyUsageTestCase(test.TestCase):
             'user_id': '158cba1b8c2bb6008e',
             'host': 'fake_host',
             'availability_zone': 'nova',
-            'volume_type': 'fake_volume_type_id',
-            'volume_id': 'fake_volume_id',
+            'volume_type': fake.VOLUME_TYPE_ID,
+            'volume_id': fake.VOLUME_ID,
             'size': 1,
             'display_name': 'test_volume',
             'created_at': '2015-01-01T01:01:01',
@@ -1206,7 +1206,6 @@ class VolumeUtilsTestCase(test.TestCase):
             volume = fake_volume.fake_volume_obj(ctxt, **volume_data)
 
             ret = volume_utils.check_encryption_provider(
-                db,
                 volume,
                 mock.sentinel.context)
             self.assertEqual('aes-xts-plain64', ret['cipher'])
@@ -1227,11 +1226,11 @@ class VolumeUtilsTestCase(test.TestCase):
             ctxt = context.get_admin_context()
             volume = fake_volume.fake_volume_obj(ctxt, **volume_data)
 
-            self.assertRaises(exception.VolumeDriverException,
-                              volume_utils.check_encryption_provider,
-                              db,
-                              volume,
-                              mock.sentinel.context)
+            self.assertRaises(
+                exception.VolumeDriverException,
+                volume_utils.check_encryption_provider,
+                volume,
+                mock.sentinel.context)
 
     @mock.patch('cinder.volume.volume_utils.CONF.list_all_sections')
     def test_get_backend_configuration_backend_stanza_not_found(self,
@@ -1265,6 +1264,16 @@ class VolumeUtilsTestCase(test.TestCase):
         mock_configuration.assert_called_with(driver.volume_opts,
                                               config_group='backendA')
 
+    def test_require_driver_initialized(self):
+        driver = mock.Mock()
+        driver.initialized = True
+        volume_utils.require_driver_initialized(driver)
+
+        driver.initialized = False
+        self.assertRaises(exception.DriverNotInitialized,
+                          volume_utils.require_driver_initialized,
+                          driver)
+
 
 @ddt.ddt
 class LogTracingTestCase(test.TestCase):
