@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2019 Dell Inc. or its subsidiaries.
+# Copyright (c) 2020 Dell Inc. or its subsidiaries.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,7 +15,8 @@
 
 from unittest import mock
 
-from cinder import test
+from cinder import exception
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.dell_emc.powermax import (
     powermax_data as tpd)
 from cinder.tests.unit.volume.drivers.dell_emc.powermax import (
@@ -34,8 +35,9 @@ class PowerMaxFCTest(test.TestCase):
         volume_utils.get_max_over_subscription_ratio = mock.Mock()
         self.configuration = tpfo.FakeConfiguration(
             None, 'FCTests', 1, 1, san_ip='1.1.1.1', san_login='smc',
-            vmax_array=self.data.array, vmax_srp='SRP_1', san_password='smc',
-            san_api_port=8443, vmax_port_groups=[self.data.port_group_name_i])
+            powermax_array=self.data.array, powermax_srp='SRP_1',
+            san_password='smc', san_api_port=8443,
+            powermax_port_groups=[self.data.port_group_name_i])
         rest.PowerMaxRest._establish_rest_session = mock.Mock(
             return_value=tpfo.FakeRequestsSession())
         driver = fc.PowerMaxFCDriver(configuration=self.configuration)
@@ -112,7 +114,8 @@ class PowerMaxFCTest(test.TestCase):
                                              self.data.connector)
             self.assertEqual(ref_data, data)
             mock_build.assert_called_once_with(
-                self.data.test_volume, self.data.connector)
+                self.data.test_volume, self.data.connector,
+                self.data.fc_device_info)
 
     def test_terminate_connection(self):
         with mock.patch.object(
@@ -187,7 +190,7 @@ class PowerMaxFCTest(test.TestCase):
             data = self.driver._cleanup_zones(self.data.zoning_mappings)
             self.assertEqual(ref_data, data)
 
-    def test_build_initiator_target_map(self):
+    def test_build_initiator_target_map_default(self):
         ref_target_map = {'123456789012345': ['543210987654321'],
                           '123456789054321': ['123450987654321']}
         with mock.patch.object(fczm_utils, 'create_lookup_service',
@@ -200,6 +203,40 @@ class PowerMaxFCTest(test.TestCase):
                     self.data.test_volume, self.data.connector)
                 self.assertEqual(ref_target_map, target_map)
 
+    def test_build_initiator_target_map_load_balanced(self):
+        init_wwns = self.data.connector.get('wwpns')
+        init_a, init_b = init_wwns[0], init_wwns[1]
+        self.driver.performance.config = self.data.performance_config
+        with mock.patch.object(
+                self.common, 'get_target_wwns_from_masking_view',
+                return_value=(self.data.target_wwns_multi, [])):
+            targets, target_map = self.driver._build_initiator_target_map(
+                self.data.test_volume, self.data.connector,
+                device_info=self.data.fc_device_info)
+            self.assertEqual(1, len(target_map.get(init_a)))
+            self.assertEqual(1, len(target_map.get(init_b)))
+            self.assertTrue(
+                len(target_map.get(init_a)) < len(self.data.target_wwns_multi))
+            self.assertTrue(
+                len(target_map.get(init_b)) < len(self.data.target_wwns_multi))
+            self.assertEqual(self.data.wwpn1, targets[0])
+
+    def test_build_initiator_target_map_load_balanced_exception(self):
+        ref_target_map = {'123456789012345': self.data.target_wwns_multi,
+                          '123456789054321': self.data.target_wwns_multi}
+        self.driver.performance.config = self.data.performance_config
+        with mock.patch.object(
+            self.common, 'get_target_wwns_from_masking_view',
+                return_value=(self.data.target_wwns_multi, [])) as mck_wwns:
+            with mock.patch.object(
+                self.driver.performance, 'process_port_load',
+                    side_effect=exception.VolumeBackendAPIException('')):
+                targets, target_map = self.driver._build_initiator_target_map(
+                    self.data.test_volume, self.data.connector,
+                    device_info=self.data.iscsi_device_info)
+                self.assertEqual(ref_target_map, target_map)
+                self.assertEqual(mck_wwns.call_count, 2)
+
     def test_extend_volume(self):
         with mock.patch.object(self.common, 'extend_volume') as mock_extend:
             self.driver.extend_volume(self.data.test_volume, '3')
@@ -207,18 +244,22 @@ class PowerMaxFCTest(test.TestCase):
 
     def test_get_volume_stats(self):
         with mock.patch.object(
-                self.driver, 'update_volume_stats') as mock_update:
-            # no refresh
-            self.driver.get_volume_stats()
-            mock_update.assert_not_called()
+                self.driver, '_update_volume_stats') as mock_update:
+
             # with refresh
             self.driver.get_volume_stats(True)
+
+            # set fake stats
+            self.driver._stats['driver_version'] = self.driver.VERSION
+            # no refresh
+            self.driver.get_volume_stats()
+
             mock_update.assert_called_once_with()
 
     def test_update_volume_stats(self):
         with mock.patch.object(self.common, 'update_volume_stats',
                                return_value={}) as mock_update:
-            self.driver.update_volume_stats()
+            self.driver._update_volume_stats()
             mock_update.assert_called_once_with()
 
     def test_check_for_setup_error(self):
