@@ -21,7 +21,7 @@ import ddt
 from oslo_utils import units
 
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit.volume.drivers.dell_emc.unity \
     import fake_enum as enums
 from cinder.tests.unit.volume.drivers.dell_emc.unity \
@@ -470,7 +470,7 @@ def patch_for_unity_adapter(func):
     @mock.patch('cinder.volume.drivers.dell_emc.unity.utils.'
                 'group_is_cg',
                 new=group_is_cg)
-    @mock.patch('cinder.utils.brick_get_connector_properties',
+    @mock.patch('cinder.volume.volume_utils.brick_get_connector_properties',
                 new=get_connector_properties)
     def func_wrapper(*args, **kwargs):
         return func(*args, **kwargs)
@@ -1847,8 +1847,8 @@ class ISCSIAdapterTest(test.TestCase):
         self.assertListEqual([hlu, hlu], info['target_luns'])
         self.assertListEqual(target_portals, info['target_portals'])
         self.assertEqual(hlu, info['target_lun'])
-        self.assertTrue(info['target_portal'] in target_portals)
-        self.assertTrue(info['target_iqn'] in target_iqns)
+        self.assertIn(info['target_portal'], target_portals)
+        self.assertIn(info['target_iqn'], target_iqns)
 
     @patch_for_iscsi_adapter
     def test_initialize_connection_volume(self):
