@@ -15,8 +15,8 @@
 
 
 from os import path
+from unittest import mock
 
-import mock
 import six
 import yaml
 
@@ -30,6 +30,9 @@ patch_sleep = mock.patch('time.sleep')
 patch_vnxsystem = mock.patch('storops.VNXSystem')
 
 
+patch_vnxstoragegroup = mock.patch('storops.vnx.resource.sg.VNXStorageGroup')
+
+
 def load_yaml(file_name):
     yaml_file = '{}/{}'.format(path.dirname(
         path.abspath(__file__)), file_name)
