@@ -1,4 +1,4 @@
-# Copyright (C) 2020, Hitachi, Ltd.
+# Copyright (C) 2020, 2021, Hitachi, Ltd.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -23,6 +23,7 @@ from requests import models
 
 from cinder import context as cinder_context
 from cinder.db.sqlalchemy import api as sqlalchemy_api
+from cinder import exception
 from cinder.objects import group_snapshot as obj_group_snap
 from cinder.objects import snapshot as obj_snap
 from cinder.tests.unit import fake_group
@@ -36,7 +37,7 @@ from cinder.volume.drivers.hitachi import hbsd_common
 from cinder.volume.drivers.hitachi import hbsd_fc
 from cinder.volume.drivers.hitachi import hbsd_rest
 from cinder.volume.drivers.hitachi import hbsd_rest_api
-from cinder.volume.drivers.hitachi import hbsd_utils
+from cinder.volume.drivers.hitachi import hbsd_rest_fc
 from cinder.volume import volume_types
 from cinder.volume import volume_utils
 from cinder.zonemanager import utils as fczm_utils
@@ -72,6 +73,14 @@ DEFAULT_CONNECTOR = {
     'multipath': False,
 }
 
+DEFAULT_CONNECTOR_AIX = {
+    'os_type': 'aix',
+    'host': 'host',
+    'ip': CONFIG_MAP['my_ip'],
+    'wwpns': [CONFIG_MAP['host_wwn']],
+    'multipath': False,
+}
+
 CTXT = cinder_context.get_admin_context()
 
 TEST_VOLUME = []
@@ -79,6 +88,7 @@ for i in range(4):
     volume = {}
     volume['id'] = '00000000-0000-0000-0000-{0:012d}'.format(i)
     volume['name'] = 'test-volume{0:d}'.format(i)
+    volume['volume_type_id'] = '00000000-0000-0000-0000-{0:012d}'.format(i)
     if i == 3:
         volume['provider_location'] = None
     else:
@@ -89,6 +99,7 @@ for i in range(4):
     else:
         volume['status'] = 'available'
     volume = fake_volume.fake_volume_obj(CTXT, **volume)
+    volume.volume_type = fake_volume.fake_volume_type_obj(CTXT)
     TEST_VOLUME.append(volume)
 
 
@@ -317,42 +328,48 @@ def _brick_get_connector_properties(multipath=False, enforce_multipath=False):
     return DEFAULT_CONNECTOR
 
 
+def _brick_get_connector_properties_aix(
+        multipath=False, enforce_multipath=False):
+    """Return a predefined connector object."""
+    return DEFAULT_CONNECTOR_AIX
+
+
 def reduce_retrying_time(func):
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
-        backup_lock_waittime = hbsd_rest_api._LOCK_WAITTIME
-        backup_exec_max_waittime = hbsd_rest_api._EXEC_MAX_WAITTIME
+        backup_lock_waittime = hbsd_rest_api._LOCK_TIMEOUT
+        backup_exec_max_waittime = hbsd_rest_api._REST_TIMEOUT
         backup_job_api_response_timeout = (
             hbsd_rest_api._JOB_API_RESPONSE_TIMEOUT)
         backup_get_api_response_timeout = (
             hbsd_rest_api._GET_API_RESPONSE_TIMEOUT)
-        backup_extend_waittime = hbsd_rest_api._EXTEND_WAITTIME
+        backup_extend_waittime = hbsd_rest_api._EXTEND_TIMEOUT
         backup_exec_retry_interval = hbsd_rest_api._EXEC_RETRY_INTERVAL
         backup_rest_server_restart_timeout = (
             hbsd_rest_api._REST_SERVER_RESTART_TIMEOUT)
-        backup_default_process_waittime = (
-            hbsd_utils.DEFAULT_PROCESS_WAITTIME)
-        hbsd_rest_api._LOCK_WAITTIME = 0.01
-        hbsd_rest_api._EXEC_MAX_WAITTIME = 0.01
+        backup_state_transition_timeout = (
+            hbsd_rest._STATE_TRANSITION_TIMEOUT)
+        hbsd_rest_api._LOCK_TIMEOUT = 0.01
+        hbsd_rest_api._REST_TIMEOUT = 0.01
         hbsd_rest_api._JOB_API_RESPONSE_TIMEOUT = 0.01
         hbsd_rest_api._GET_API_RESPONSE_TIMEOUT = 0.01
-        hbsd_rest_api._EXTEND_WAITTIME = 0.01
+        hbsd_rest_api._EXTEND_TIMEOUT = 0.01
         hbsd_rest_api._EXEC_RETRY_INTERVAL = 0.004
         hbsd_rest_api._REST_SERVER_RESTART_TIMEOUT = 0.02
-        hbsd_utils.DEFAULT_PROCESS_WAITTIME = 0.01
+        hbsd_rest._STATE_TRANSITION_TIMEOUT = 0.01
         func(*args, **kwargs)
-        hbsd_rest_api._LOCK_WAITTIME = backup_lock_waittime
-        hbsd_rest_api._EXEC_MAX_WAITTIME = backup_exec_max_waittime
+        hbsd_rest_api._LOCK_TIMEOUT = backup_lock_waittime
+        hbsd_rest_api._REST_TIMEOUT = backup_exec_max_waittime
         hbsd_rest_api._JOB_API_RESPONSE_TIMEOUT = (
             backup_job_api_response_timeout)
         hbsd_rest_api._GET_API_RESPONSE_TIMEOUT = (
             backup_get_api_response_timeout)
-        hbsd_rest_api._EXTEND_WAITTIME = backup_extend_waittime
+        hbsd_rest_api._EXTEND_TIMEOUT = backup_extend_waittime
         hbsd_rest_api._EXEC_RETRY_INTERVAL = backup_exec_retry_interval
         hbsd_rest_api._REST_SERVER_RESTART_TIMEOUT = (
             backup_rest_server_restart_timeout)
-        hbsd_utils.DEFAULT_PROCESS_WAITTIME = (
-            backup_default_process_waittime)
+        hbsd_rest._STATE_TRANSITION_TIMEOUT = (
+            backup_state_transition_timeout)
     return wrapper
 
 
@@ -428,6 +445,9 @@ class HBSDRESTFCDriverTest(test.TestCase):
             CONFIG_MAP['port_id']]
         self.configuration.hitachi_group_create = True
         self.configuration.hitachi_group_delete = True
+        self.configuration.hitachi_copy_speed = 3
+        self.configuration.hitachi_copy_check_interval = 3
+        self.configuration.hitachi_async_copy_check_interval = 10
 
         self.configuration.san_login = CONFIG_MAP['user_id']
         self.configuration.san_password = CONFIG_MAP['user_pass']
@@ -435,9 +455,41 @@ class HBSDRESTFCDriverTest(test.TestCase):
             'rest_server_ip_addr']
         self.configuration.san_api_port = CONFIG_MAP[
             'rest_server_ip_port']
+        self.configuration.hitachi_rest_disable_io_wait = True
         self.configuration.hitachi_rest_tcp_keepalive = True
         self.configuration.hitachi_discard_zero_page = True
         self.configuration.hitachi_rest_number = "0"
+        self.configuration.hitachi_lun_timeout = hbsd_rest._LUN_TIMEOUT
+        self.configuration.hitachi_lun_retry_interval = (
+            hbsd_rest._LUN_RETRY_INTERVAL)
+        self.configuration.hitachi_restore_timeout = hbsd_rest._RESTORE_TIMEOUT
+        self.configuration.hitachi_state_transition_timeout = (
+            hbsd_rest._STATE_TRANSITION_TIMEOUT)
+        self.configuration.hitachi_lock_timeout = hbsd_rest_api._LOCK_TIMEOUT
+        self.configuration.hitachi_rest_timeout = hbsd_rest_api._REST_TIMEOUT
+        self.configuration.hitachi_extend_timeout = (
+            hbsd_rest_api._EXTEND_TIMEOUT)
+        self.configuration.hitachi_exec_retry_interval = (
+            hbsd_rest_api._EXEC_RETRY_INTERVAL)
+        self.configuration.hitachi_rest_connect_timeout = (
+            hbsd_rest_api._DEFAULT_CONNECT_TIMEOUT)
+        self.configuration.hitachi_rest_job_api_response_timeout = (
+            hbsd_rest_api._JOB_API_RESPONSE_TIMEOUT)
+        self.configuration.hitachi_rest_get_api_response_timeout = (
+            hbsd_rest_api._GET_API_RESPONSE_TIMEOUT)
+        self.configuration.hitachi_rest_server_busy_timeout = (
+            hbsd_rest_api._REST_SERVER_BUSY_TIMEOUT)
+        self.configuration.hitachi_rest_keep_session_loop_interval = (
+            hbsd_rest_api._KEEP_SESSION_LOOP_INTERVAL)
+        self.configuration.hitachi_rest_another_ldev_mapped_retry_timeout = (
+            hbsd_rest_api._ANOTHER_LDEV_MAPPED_RETRY_TIMEOUT)
+        self.configuration.hitachi_rest_tcp_keepidle = (
+            hbsd_rest_api._TCP_KEEPIDLE)
+        self.configuration.hitachi_rest_tcp_keepintvl = (
+            hbsd_rest_api._TCP_KEEPINTVL)
+        self.configuration.hitachi_rest_tcp_keepcnt = (
+            hbsd_rest_api._TCP_KEEPCNT)
+        self.configuration.hitachi_host_mode_options = []
 
         self.configuration.hitachi_zoning_request = False
 
@@ -543,6 +595,36 @@ class HBSDRESTFCDriverTest(test.TestCase):
         self.driver.common.client.keep_session_loop.stop()
         self.driver.common.client.keep_session_loop.wait()
 
+    @mock.patch.object(requests.Session, "request")
+    @mock.patch.object(
+        volume_utils, 'brick_get_connector_properties',
+        side_effect=_brick_get_connector_properties_aix)
+    def test_do_setup_create_hg_aix(
+            self, brick_get_connector_properties, request):
+        """Normal case: The host group not exists in AIX."""
+        drv = hbsd_fc.HBSDFCDriver(
+            configuration=self.configuration)
+        self._setup_config()
+        request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
+                               FakeResponse(200, GET_PORTS_RESULT),
+                               FakeResponse(200, NOTFOUND_RESULT),
+                               FakeResponse(200, NOTFOUND_RESULT),
+                               FakeResponse(200, NOTFOUND_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        drv.do_setup(None)
+        self.assertEqual(
+            {CONFIG_MAP['port_id']: CONFIG_MAP['target_wwn']},
+            drv.common.storage_info['wwns'])
+        self.assertEqual(1, brick_get_connector_properties.call_count)
+        self.assertEqual(8, request.call_count)
+        kargs1 = request.call_args_list[6][1]
+        self.assertEqual('AIX', kargs1['json']['hostMode'])
+        # stop the Loopingcall within the do_setup treatment
+        self.driver.common.client.keep_session_loop.stop()
+        self.driver.common.client.keep_session_loop.wait()
+
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
         volume_utils, 'brick_get_connector_properties',
@@ -582,7 +664,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
         request.return_value = FakeResponse(
             500, ERROR_RESULT,
             headers={'Content-Type': 'json'})
-        self.assertRaises(hbsd_utils.HBSDError,
+        self.assertRaises(exception.VolumeDriverException,
                           self.driver.create_volume,
                           fake_volume.fake_volume_obj(self.ctxt))
         self.assertGreater(request.call_count, 1)
@@ -616,7 +698,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
                                FakeResponse(200, GET_LDEV_RESULT_PAIR),
                                FakeResponse(200, GET_LDEV_RESULT_PAIR),
                                FakeResponse(200, GET_LDEV_RESULT_PAIR)]
-        self.assertRaises(hbsd_utils.HBSDError,
+        self.assertRaises(exception.VolumeDriverException,
                           self.driver.delete_volume,
                           TEST_VOLUME[0])
         self.assertGreater(request.call_count, 2)
@@ -705,9 +787,13 @@ class HBSDRESTFCDriverTest(test.TestCase):
 
     @mock.patch.object(fczm_utils, "add_fc_zone")
     @mock.patch.object(requests.Session, "request")
-    def test_initialize_connection(self, request, add_fc_zone):
-        self.configuration.hitachi_zoning_request = True
+    @mock.patch.object(volume_types, 'get_volume_type_extra_specs')
+    def test_initialize_connection(
+            self, get_volume_type_extra_specs, request, add_fc_zone):
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
+        extra_specs = {"hbsd:target_ports": "CL1-A"}
+        get_volume_type_extra_specs.return_value = extra_specs
         request.side_effect = [FakeResponse(200, GET_HOST_WWNS_RESULT),
                                FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
         ret = self.driver.initialize_connection(
@@ -715,15 +801,20 @@ class HBSDRESTFCDriverTest(test.TestCase):
         self.assertEqual('fibre_channel', ret['driver_volume_type'])
         self.assertEqual([CONFIG_MAP['target_wwn']], ret['data']['target_wwn'])
         self.assertEqual(1, ret['data']['target_lun'])
+        self.assertEqual(1, get_volume_type_extra_specs.call_count)
         self.assertEqual(2, request.call_count)
         self.assertEqual(1, add_fc_zone.call_count)
 
     @mock.patch.object(fczm_utils, "add_fc_zone")
     @mock.patch.object(requests.Session, "request")
-    def test_initialize_connection_already_mapped(self, request, add_fc_zone):
+    @mock.patch.object(volume_types, 'get_volume_type_extra_specs')
+    def test_initialize_connection_already_mapped(
+            self, get_volume_type_extra_specs, request, add_fc_zone):
         """Normal case: ldev have already mapped."""
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
+        extra_specs = {"hbsd:target_ports": "CL1-A"}
+        get_volume_type_extra_specs.return_value = extra_specs
         request.side_effect = [
             FakeResponse(200, GET_HOST_WWNS_RESULT),
             FakeResponse(202, COMPLETED_FAILED_RESULT_LU_DEFINED),
@@ -734,15 +825,20 @@ class HBSDRESTFCDriverTest(test.TestCase):
         self.assertEqual('fibre_channel', ret['driver_volume_type'])
         self.assertEqual([CONFIG_MAP['target_wwn']], ret['data']['target_wwn'])
         self.assertEqual(1, ret['data']['target_lun'])
+        self.assertEqual(1, get_volume_type_extra_specs.call_count)
         self.assertEqual(3, request.call_count)
         self.assertEqual(1, add_fc_zone.call_count)
 
     @mock.patch.object(fczm_utils, "add_fc_zone")
     @mock.patch.object(requests.Session, "request")
-    def test_initialize_connection_shared_target(self, request, add_fc_zone):
+    @mock.patch.object(volume_types, 'get_volume_type_extra_specs')
+    def test_initialize_connection_shared_target(
+            self, get_volume_type_extra_specs, request, add_fc_zone):
         """Normal case: A target shared with other systems."""
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
+        extra_specs = {"hbsd:target_ports": "CL1-A"}
+        get_volume_type_extra_specs.return_value = extra_specs
         request.side_effect = [FakeResponse(200, NOTFOUND_RESULT),
                                FakeResponse(200, NOTFOUND_RESULT),
                                FakeResponse(200, GET_HOST_GROUPS_RESULT),
@@ -753,13 +849,14 @@ class HBSDRESTFCDriverTest(test.TestCase):
         self.assertEqual('fibre_channel', ret['driver_volume_type'])
         self.assertEqual([CONFIG_MAP['target_wwn']], ret['data']['target_wwn'])
         self.assertEqual(1, ret['data']['target_lun'])
+        self.assertEqual(1, get_volume_type_extra_specs.call_count)
         self.assertEqual(5, request.call_count)
         self.assertEqual(1, add_fc_zone.call_count)
 
     @mock.patch.object(fczm_utils, "remove_fc_zone")
     @mock.patch.object(requests.Session, "request")
     def test_terminate_connection(self, request, remove_fc_zone):
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
         request.side_effect = [FakeResponse(200, GET_HOST_WWNS_RESULT),
                                FakeResponse(200, GET_LDEV_RESULT_MAPPED),
@@ -774,7 +871,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
     @mock.patch.object(requests.Session, "request")
     def test_terminate_connection_not_connector(self, request, remove_fc_zone):
         """Normal case: Connector is None."""
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
         request.side_effect = [FakeResponse(200, GET_LDEV_RESULT_MAPPED),
                                FakeResponse(200, GET_HOST_GROUP_RESULT),
@@ -792,7 +889,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
     @mock.patch.object(requests.Session, "request")
     def test_terminate_connection_not_lun(self, request, remove_fc_zone):
         """Normal case: Lun already not exist."""
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
         request.side_effect = [FakeResponse(200, GET_HOST_WWNS_RESULT),
                                FakeResponse(200, GET_LDEV_RESULT)]
@@ -803,7 +900,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
     @mock.patch.object(fczm_utils, "add_fc_zone")
     @mock.patch.object(requests.Session, "request")
     def test_initialize_connection_snapshot(self, request, add_fc_zone):
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
         request.side_effect = [FakeResponse(200, GET_HOST_WWNS_RESULT),
                                FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
@@ -818,7 +915,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
     @mock.patch.object(fczm_utils, "remove_fc_zone")
     @mock.patch.object(requests.Session, "request")
     def test_terminate_connection_snapshot(self, request, remove_fc_zone):
-        self.configuration.hitachi_zoning_request = True
+        self.driver.common.conf.hitachi_zoning_request = True
         self.driver.common._lookup_service = FakeLookupService()
         request.side_effect = [FakeResponse(200, GET_HOST_WWNS_RESULT),
                                FakeResponse(200, GET_LDEV_RESULT_MAPPED),
@@ -985,7 +1082,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
 
     def test_create_group_from_src_volume_error(self):
         self.assertRaises(
-            hbsd_utils.HBSDError, self.driver.create_group_from_src,
+            exception.VolumeDriverException, self.driver.create_group_from_src,
             self.ctxt, TEST_GROUP[1], [TEST_VOLUME[1]],
             source_group=TEST_GROUP[0], source_vols=[TEST_VOLUME[3]]
         )
@@ -1001,7 +1098,7 @@ class HBSDRESTFCDriverTest(test.TestCase):
     def test_update_group_error(self, is_group_a_cg_snapshot_type):
         is_group_a_cg_snapshot_type.return_value = True
         self.assertRaises(
-            hbsd_utils.HBSDError, self.driver.update_group,
+            exception.VolumeDriverException, self.driver.update_group,
             self.ctxt, TEST_GROUP[0], add_volumes=[TEST_VOLUME[3]],
             remove_volumes=[TEST_VOLUME[0]]
         )
@@ -1071,3 +1168,12 @@ class HBSDRESTFCDriverTest(test.TestCase):
             [{'id': TEST_SNAPSHOT[0]['id'], 'status': 'deleted'}]
         )
         self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(hbsd_fc.HBSDFCDriver, "_get_oslo_driver_opts")
+    def test_get_driver_options(self, _get_oslo_driver_opts):
+        _get_oslo_driver_opts.return_value = []
+        ret = self.driver.get_driver_options()
+        actual = (hbsd_common.COMMON_VOLUME_OPTS +
+                  hbsd_rest.REST_VOLUME_OPTS +
+                  hbsd_rest_fc.FC_VOLUME_OPTS)
+        self.assertEqual(actual, ret)
