@@ -13,32 +13,27 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Volume driver test for Inspur AS13000
-"""
+"""Volume driver test for Inspur AS13000."""
 
 import json
-import mock
 import random
 import time
+from unittest import mock
 
 import ddt
 import eventlet
-from oslo_config import cfg
 import requests
 
 from cinder import context
 from cinder import exception
-from cinder import test
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit import test
 from cinder.volume import configuration
 from cinder.volume.drivers.inspur.as13000 import as13000_driver
 from cinder.volume import volume_utils
 
 
-CONF = cfg.CONF
-
 test_config = configuration.Configuration(None)
 test_config.san_ip = 'some_ip'
 test_config.san_api_port = 'as13000_api_port'
@@ -997,7 +992,7 @@ class AS13000DriverTestCase(test.TestCase):
         host_exist, target_name, node = (
             self.as13000_san._get_target_from_conn(host_ip))
 
-        if host_ip is 'fake_ip3':
+        if host_ip == 'fake_ip3':
             self.assertEqual((True, 'fake_target_2',
                               ['fake_node4', 'fake_node3']),
                              (host_exist, target_name, node))
