@@ -13,17 +13,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-"""
-Tests for the Inspur InStorage volume driver.
-"""
+"""Tests for the Inspur InStorage volume driver."""
+
+from unittest import mock
 
 from eventlet import greenthread
-import mock
-from oslo_utils import importutils
 
 from cinder import context
+import cinder.db
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.tests.unit import utils as testutils
 from cinder.tests.unit.volume.drivers.inspur.instorage import fakes
 from cinder.volume import configuration as conf
@@ -59,8 +58,7 @@ class InStorageMCSFcDriverTestCase(test.TestCase):
 
         self._reset_flags()
         self.ctxt = context.get_admin_context()
-        db_driver = self.fc_driver.configuration.db_driver
-        self.db = importutils.import_module(db_driver)
+        self.db = cinder.db
         self.fc_driver.db = self.db
         self.fc_driver.do_setup(None)
         self.fc_driver.check_for_setup_error()
@@ -414,7 +412,7 @@ class InStorageMCSFcDriverTestCase(test.TestCase):
                               }
                      }
 
-        self.assertItemsEqual(term_data, term_ret)
+        self.assertCountEqual(term_data, term_ret)
 
     def test_instorage_mcs_fc_host_maps(self):
         # Create two volumes to be used in mappings
@@ -595,3 +593,16 @@ class InStorageMCSFcDriverTestCase(test.TestCase):
         volume = self._generate_vol_info(None, None)
         self.fc_driver.create_volume(volume)
         self.fc_driver.add_vdisk_copy(volume['name'], 'fake-pool', None)
+
+    def test_make_initiator_target_all2all_map(self):
+        initiator_wwpns = ['ff00000000000000', 'ff00000000000001']
+        target_wwpns = ['bb00000000000000', 'bb00000000000001']
+
+        expected = {
+            'ff00000000000000': ['bb00000000000000', 'bb00000000000001'],
+            'ff00000000000001': ['bb00000000000000', 'bb00000000000001']
+        }
+
+        ret = self.fc_driver.make_initiator_target_all2all_map(initiator_wwpns,
+                                                               target_wwpns)
+        self.assertEqual(ret, expected)
