@@ -19,14 +19,14 @@ import json
 from unittest import mock
 
 from eventlet import greenthread
-from oslo_utils import importutils
 from oslo_utils import units
 
 from cinder import context
+import cinder.db
 from cinder import exception
 from cinder.objects import fields
-from cinder import test
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 from cinder.tests.unit import utils as testutils
 from cinder.tests.unit.volume.drivers.inspur.instorage import fakes
 from cinder import utils
@@ -86,9 +86,7 @@ class InStorageMCSReplicationTestCase(test.TestCase):
 
         self._reset_flags()
         self.ctxt = context.get_admin_context()
-        db_driver = self.driver.configuration.db_driver
-        self.db = importutils.import_module(db_driver)
-        self.driver.db = self.db
+        self.driver.db = cinder.db
 
         self.driver.do_setup(None)
         self.driver.check_for_setup_error()
