@@ -19,7 +19,7 @@ import uuid
 
 from cinder import context
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder import version
 from cinder.volume import configuration as conf
 from cinder.volume import volume_types
