@@ -16,12 +16,12 @@
 
 
 """Unit tests for Cisco fc san lookup service."""
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.volume import configuration as conf
 import cinder.zonemanager.drivers.cisco.cisco_fc_san_lookup_service \
     as cisco_lookup
