@@ -130,10 +130,9 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
               - Use of snap id instead of generation (bp powermax-snapset-ids)
               - Support for Failover Abilities (bp/powermax-failover-abilities)
         4.4.0 - Early check for status of port
-        4.4.1 - Allow for case mismatch in SGs (bug #1929429)
     """
 
-    VERSION = "4.4.1"
+    VERSION = "4.4.0"
 
     # ThirdPartySystems wiki
     CI_WIKI_NAME = "DellEMC_PowerMAX_CI"
@@ -497,7 +496,7 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
                 # Get target WWN
                 port_details = self.rest.get_port(array_id, port)
                 port_info = port_details.get('symmetrixPort')
-                port_wwn = port_info.get('wwn_node')
+                port_wwn = port_info.get('identifier')
                 LOG.info("Port %(p)s WWN: %(wwn)s",
                          {'p': port, 'wwn': port_wwn})
                 # Set lowest load port WWN as FC target for connection
