@@ -1,4 +1,4 @@
-# Copyright (C) 2020, Hitachi, Ltd.
+# Copyright (C) 2020, 2021, Hitachi, Ltd.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -17,30 +17,39 @@ REST API client class for Hitachi HBSD Driver.
 """
 
 from http import client as httpclient
+import socket
 import threading
 
 from eventlet import greenthread
-from keystoneauth1.session import TCPKeepAliveAdapter
 from oslo_log import log as logging
 from oslo_service import loopingcall
 from oslo_utils import timeutils
 import requests
+from requests.adapters import HTTPAdapter
+from requests.packages.urllib3.connection import HTTPConnection
+from requests.packages.urllib3.poolmanager import PoolManager
 
+from cinder import exception
+from cinder.i18n import _
 from cinder.volume.drivers.hitachi import hbsd_utils as utils
 from cinder.volume import volume_utils
 
-_LOCK_WAITTIME = 2 * 60 * 60
-_EXEC_MAX_WAITTIME = 30
-_EXTEND_WAITTIME = 10 * 60
+_LOCK_TIMEOUT = 2 * 60 * 60
+_REST_TIMEOUT = 30
+_EXTEND_TIMEOUT = 10 * 60
 _EXEC_RETRY_INTERVAL = 5
 _DEFAULT_CONNECT_TIMEOUT = 30
-_RESPONSE_TIMEOUT_TOLERANCE = 30
 _JOB_API_RESPONSE_TIMEOUT = 30 * 60
 _GET_API_RESPONSE_TIMEOUT = 30 * 60
 _REST_SERVER_BUSY_TIMEOUT = 2 * 60 * 60
 _REST_SERVER_RESTART_TIMEOUT = 10 * 60
 _REST_SERVER_ERROR_TIMEOUT = 10 * 60
 _KEEP_SESSION_LOOP_INTERVAL = 3 * 60
+_ANOTHER_LDEV_MAPPED_RETRY_TIMEOUT = 10 * 60
+
+_TCP_KEEPIDLE = 60
+_TCP_KEEPINTVL = 15
+_TCP_KEEPCNT = 4
 
 _HTTPS = 'https://'
 
@@ -89,6 +98,28 @@ def _build_base_url(ip_addr, ip_port):
     }
 
 
+class KeepAliveAdapter(HTTPAdapter):
+
+    def __init__(self, conf):
+        self.options = HTTPConnection.default_socket_options + [
+            (socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1),
+            (socket.IPPROTO_TCP, socket.TCP_KEEPIDLE,
+             conf.hitachi_rest_tcp_keepidle),
+            (socket.IPPROTO_TCP, socket.TCP_KEEPINTVL,
+             conf.hitachi_rest_tcp_keepintvl),
+            (socket.IPPROTO_TCP, socket.TCP_KEEPCNT,
+             conf.hitachi_rest_tcp_keepcnt),
+        ]
+
+        super(KeepAliveAdapter, self).__init__()
+
+    def init_poolmanager(self, connections, maxsize, block=False):
+        self.poolmanager = PoolManager(num_pools=connections,
+                                       maxsize=maxsize,
+                                       block=block,
+                                       socket_options=self.options)
+
+
 class ResponseData(dict):
 
     def is_json(self):
@@ -193,10 +224,11 @@ class ResponseData(dict):
 
 class RestApiClient():
 
-    def __init__(self, ip_addr, ip_port, storage_device_id,
-                 user_id, user_pass, tcp_keepalive=False,
-                 verify=False, connect_timeout=_DEFAULT_CONNECT_TIMEOUT):
+    def __init__(self, conf, ip_addr, ip_port, storage_device_id,
+                 user_id, user_pass, driver_prefix, tcp_keepalive=False,
+                 verify=False):
         """Initialize instance variables."""
+        self.conf = conf
         self.ip_addr = ip_addr
         self.ip_port = ip_port
         self.storage_id = storage_device_id
@@ -205,7 +237,7 @@ class RestApiClient():
         self.user_pass = user_pass
         self.tcp_keepalive = tcp_keepalive
         self.verify = verify
-        self.connect_timeout = connect_timeout
+        self.connect_timeout = self.conf.hitachi_rest_connect_timeout
         self.login_lock = threading.Lock()
         self.keep_session_loop = loopingcall.FixedIntervalLoopingCall(
             self._keep_session)
@@ -221,6 +253,7 @@ class RestApiClient():
         }
         self.headers = {"content-type": "application/json",
                         "accept": "application/json"}
+        self.driver_prefix = driver_prefix
 
     class Session(requests.auth.AuthBase):
 
@@ -250,23 +283,24 @@ class RestApiClient():
         kwargs.setdefault('ignore_all_errors', False)
         kwargs.setdefault('timeout_message', None)
         kwargs.setdefault('no_log', False)
-        kwargs.setdefault('timeout', _EXEC_MAX_WAITTIME)
+        kwargs.setdefault('timeout', self.conf.hitachi_rest_timeout)
 
         headers = dict(self.headers)
         if async_:
-            read_timeout = (_JOB_API_RESPONSE_TIMEOUT +
-                            _RESPONSE_TIMEOUT_TOLERANCE)
+            read_timeout = self.conf.hitachi_rest_job_api_response_timeout
             headers.update({
-                "Response-Max-Wait": str(_JOB_API_RESPONSE_TIMEOUT),
+                "Response-Max-Wait": str(
+                    self.conf.hitachi_rest_job_api_response_timeout),
                 "Response-Job-Status": "Completed;"})
         else:
-            read_timeout = _GET_API_RESPONSE_TIMEOUT
+            read_timeout = self.conf.hitachi_rest_get_api_response_timeout
 
         auth_data = kwargs.get('auth', self.get_my_session())
 
         timeout = (self.connect_timeout, read_timeout)
 
-        interval = kwargs.get('interval', _EXEC_RETRY_INTERVAL)
+        interval = kwargs.get(
+            'interval', self.conf.hitachi_exec_retry_interval)
         retry = True
         start_time = timeutils.utcnow()
         watch = timeutils.StopWatch()
@@ -276,7 +310,7 @@ class RestApiClient():
             try:
                 with requests.Session() as session:
                     if self.tcp_keepalive:
-                        session.mount(_HTTPS, TCPKeepAliveAdapter())
+                        session.mount(_HTTPS, KeepAliveAdapter(self.conf))
                     rsp = session.request(method, url,
                                           params=params,
                                           json=body,
@@ -290,7 +324,13 @@ class RestApiClient():
                     MSG.REST_SERVER_CONNECT_FAILED,
                     exception=type(e), message=e,
                     method=method, url=url, params=params, body=body)
-                raise utils.HBSDError(msg)
+                message = _(
+                    '%(prefix)s error occurred. %(msg)s' % {
+                        'prefix': self.driver_prefix,
+                        'msg': msg,
+                    }
+                )
+                raise exception.VolumeDriverException(message)
 
             response = ResponseData(rsp)
             if (response['status_code'] == httpclient.INTERNAL_SERVER_ERROR and
@@ -319,14 +359,22 @@ class RestApiClient():
         errobj = response['errobj']
         if response.is_locked():
             if (kwargs['no_retry'] or
-                    utils.timed_out(start_time, _LOCK_WAITTIME)):
+                    utils.timed_out(
+                        start_time, self.conf.hitachi_lock_timeout)):
                 msg = utils.output_log(MSG.REST_API_FAILED,
                                        no_log=kwargs['no_log'],
                                        method=method, url=url,
                                        params=params, body=body,
                                        **response.get_errobj())
                 if kwargs['do_raise']:
-                    raise utils.HBSDError(msg, errobj=errobj)
+                    message = _(
+                        '%(prefix)s error occurred. %(msg)s' % {
+                            'prefix': self.driver_prefix,
+                            'msg': msg,
+                        }
+                    )
+                    raise exception.VolumeDriverException(
+                        message, errobj=errobj)
                 return False, rsp_body, errobj
             else:
                 LOG.debug("The resource group to which the operation object ",
@@ -349,7 +397,15 @@ class RestApiClient():
             retry = True
 
         if retry and response.is_rest_server_busy():
-            if utils.timed_out(start_time, _REST_SERVER_BUSY_TIMEOUT):
+            if utils.timed_out(
+                    start_time, self.conf.hitachi_rest_server_busy_timeout):
+                retry = False
+        elif retry and response.get_err_code() in (ANOTHER_LDEV_MAPPED, ):
+            if utils.timed_out(
+                    start_time,
+                    self.conf.hitachi_rest_another_ldev_mapped_retry_timeout):
+                LOG.debug(
+                    "Another LDEV is already mapped to the specified LUN.")
                 retry = False
         elif retry and utils.timed_out(start_time, kwargs['timeout']):
             if kwargs['timeout_message']:
@@ -375,7 +431,14 @@ class RestApiClient():
                                        method=method, url=url,
                                        params=params, body=body)
             if kwargs['do_raise']:
-                raise utils.HBSDError(msg, errobj=errobj)
+                message = _(
+                    '%(prefix)s error occurred. %(msg)s' % {
+                        'prefix': self.driver_prefix,
+                        'msg': msg,
+                    }
+                )
+                raise exception.VolumeDriverException(
+                    message, errobj=errobj)
             return False, rsp_body, errobj
 
         if errobj:
@@ -398,7 +461,14 @@ class RestApiClient():
                                        method=method, url=url,
                                        params=params, body=body)
             if kwargs['do_raise']:
-                raise utils.HBSDError(msg, errobj=errobj)
+                message = _(
+                    '%(prefix)s error occurred. %(msg)s' % {
+                        'prefix': self.driver_prefix,
+                        'msg': msg,
+                    }
+                )
+                raise exception.VolumeDriverException(
+                    message, errobj=errobj)
         return retry, rsp_body, errobj
 
     def set_my_session(self, session):
@@ -414,7 +484,8 @@ class RestApiClient():
         }
         auth = (self.user_id, self.user_pass)
         rsp, err = self._request("POST", url, auth=auth, no_relogin=True,
-                                 do_raise=do_raise, timeout=_LOCK_WAITTIME)
+                                 do_raise=do_raise,
+                                 timeout=self.conf.hitachi_lock_timeout)
         if not err:
             self.set_my_session(self.Session(rsp["sessionId"], rsp["token"]))
             return True
@@ -442,7 +513,7 @@ class RestApiClient():
             if session is not None:
                 self.get_session(session.id, no_retry=True, no_log=True)
                 has_session = True
-        except utils.HBSDError as ex:
+        except exception.VolumeDriverException as ex:
             LOG.debug('Failed to get session info: %s', ex)
         return has_session
 
@@ -472,7 +543,8 @@ class RestApiClient():
 
     def enter_keep_session(self):
         """Begin the keeping of a session."""
-        self.keep_session_loop.start(_KEEP_SESSION_LOOP_INTERVAL)
+        self.keep_session_loop.start(
+            self.conf.hitachi_rest_keep_session_loop_interval)
         LOG.debug('enter_keep_session')
 
     def _get_object(self, url, params=None, **kwargs):
@@ -566,7 +638,7 @@ class RestApiClient():
             'id': ldev_id,
             'action': 'expand',
         }
-        self._invoke(url, body=body, timeout=_EXTEND_WAITTIME)
+        self._invoke(url, body=body, timeout=self.conf.hitachi_extend_timeout)
 
     def get_ports(self, params=None):
         """Get a list of port information."""
@@ -750,6 +822,14 @@ class RestApiClient():
         }
         self._invoke(url, body=body)
 
+    def split_snapshotgroup(self, snapshot_group_id):
+        url = '%(url)s/snapshot-groups/%(id)s/actions/%(action)s/invoke' % {
+            'url': self.object_url,
+            'id': snapshot_group_id,
+            'action': 'split',
+        }
+        self._invoke(url)
+
     def discard_zero_page(self, ldev_id):
         """Return the ldev's no-data pages to the storage pool."""
         url = '%(url)s/ldevs/%(id)s/actions/%(action)s/invoke' % {
