@@ -41,8 +41,8 @@ from cinder import utils
 from cinder.volume import configuration
 from cinder.volume import driver
 from cinder.volume.drivers.san import san
-from cinder.volume import utils as volume_utils
 from cinder.volume import volume_types
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -97,6 +97,9 @@ class FlashSystemDriver(san.SanDriver,
 
     VERSION = "1.0.12"
 
+    # TODO(jsbryant) Remove driver in the 'U' release if CI is not fixed.
+    SUPPORTED = False
+
     MULTI_HOST_MAP_ERRORS = ['CMMVC6045E', 'CMMVC6071E']
 
     def __init__(self, *args, **kwargs):
@@ -216,7 +219,7 @@ class FlashSystemDriver(san.SanDriver,
         LOG.debug('enter: _copy_vdisk_data: %(src)s -> %(dest)s.',
                   {'src': src_vdisk_name, 'dest': dest_vdisk_name})
 
-        connector = utils.brick_get_connector_properties()
+        connector = volume_utils.brick_get_connector_properties()
         (src_map, src_lun_id) = self._is_vdisk_map(
             src_vdisk_name, connector)
         (dest_map, dest_lun_id) = self._is_vdisk_map(
@@ -849,11 +852,12 @@ class FlashSystemDriver(san.SanDriver,
         use_multipath = self.configuration.use_multipath_for_image_xfer
         device_scan_attempts = self.configuration.num_volume_device_scan_tries
         protocol = properties['driver_volume_type']
-        connector = utils.brick_get_connector(protocol,
-                                              use_multipath=use_multipath,
-                                              device_scan_attempts=
-                                              device_scan_attempts,
-                                              conn=properties)
+        connector = volume_utils.brick_get_connector(
+            protocol,
+            use_multipath=use_multipath,
+            device_scan_attempts=
+            device_scan_attempts,
+            conn=properties)
 
         connector.disconnect_volume(properties['data'], device)
 
@@ -885,11 +889,12 @@ class FlashSystemDriver(san.SanDriver,
         use_multipath = self.configuration.use_multipath_for_image_xfer
         device_scan_attempts = self.configuration.num_volume_device_scan_tries
         protocol = properties['driver_volume_type']
-        connector = utils.brick_get_connector(protocol,
-                                              use_multipath=use_multipath,
-                                              device_scan_attempts=
-                                              device_scan_attempts,
-                                              conn=properties)
+        connector = volume_utils.brick_get_connector(
+            protocol,
+            use_multipath=use_multipath,
+            device_scan_attempts=
+            device_scan_attempts,
+            conn=properties)
         device = connector.connect_volume(properties['data'])
         host_device = device['path']
 
@@ -1184,17 +1189,6 @@ class FlashSystemDriver(san.SanDriver,
         LOG.debug('leave: create_cloned_volume: create %(vol)s from %(src)s.',
                   {'src': src_volume['name'], 'vol': volume['name']})
 
-    def get_volume_stats(self, refresh=False):
-        """Get volume stats.
-
-        If we haven't gotten stats yet or 'refresh' is True,
-        run update the stats first.
-        """
-        if not self._stats or refresh:
-            self._update_volume_stats()
-
-        return self._stats
-
     def manage_existing(self, volume, existing_ref):
         """Manages an existing vdisk.
 
