@@ -16,13 +16,13 @@
 import collections
 import copy
 import distutils.version as dist_version  # pylint: disable=E0611
-import eventlet
 import math
 import os
-import six
 import string
 
+import eventlet
 from oslo_log import log as logging
+import six
 
 from cinder import coordination
 from cinder import exception
@@ -61,10 +61,10 @@ class DS8KCommonHelper(object):
 
     OPTIONAL_PARAMS = ['ds8k_host_type', 'lss_range_for_cg']
     # if use new REST API, please update the version below
-    VALID_REST_VERSION_5_7_MIN = '5.7.51.1047'
+    VALID_REST_VERSION_87_51_MIN = '87.51.52.0'
     INVALID_STORAGE_VERSION = '8.0.1'
-    REST_VERSION_5_7_MIN_PPRC_CG = '5.7.51.1068'
-    REST_VERSION_5_8_MIN_PPRC_CG = '5.8.20.1059'
+    REST_VERSION_87_51_MIN_PPRC_CG = '87.51.63.0'
+    REST_VERSION_88_20_MIN_PPRC_CG = '88.20.112.0'
 
     def __init__(self, conf, HTTPConnectorObject=None):
         self.conf = conf
@@ -148,7 +148,7 @@ class DS8KCommonHelper(object):
         except restclient.TimeoutException:
             raise restclient.APIException(
                 data=(_("Can't connect to %(host)s") % {'host': san_ip}))
-        self.backend['rest_version'] = self._get_version()['bundle_version']
+        self.backend['rest_version'] = self._get_version()['bundle']
         LOG.info("Connection to DS8K storage system %(host)s has been "
                  "established successfully, the version of REST is %(rest)s.",
                  {'host': self._get_value('san_ip'),
@@ -175,7 +175,7 @@ class DS8KCommonHelper(object):
                           'the end.'))
             lss_range = lss_range.replace('-', ' - ').split()
             for index, lss in enumerate(lss_range):
-                if lss is '-':
+                if lss == '-':
                     try:
                         begin = int(lss_range[index - 1], 16)
                         end = int(lss_range[index + 1], 16)
@@ -224,14 +224,15 @@ class DS8KCommonHelper(object):
                            "if you want to use this version of driver, "
                            "please upgrade the CCL.")
                          % self.INVALID_STORAGE_VERSION))
-        if ('5.7' in self.backend['rest_version'] and
+        rest_ver = self.backend['rest_version'][0:2]
+        if ('87' == rest_ver and
            dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.VALID_REST_VERSION_5_7_MIN)):
+           dist_version.LooseVersion(self.VALID_REST_VERSION_87_51_MIN)):
             raise exception.VolumeDriverException(
                 message=(_("REST version %(invalid)s is lower than "
                            "%(valid)s, please upgrade it in DS8K.")
                          % {'invalid': self.backend['rest_version'],
-                            'valid': self.VALID_REST_VERSION_5_7_MIN}))
+                            'valid': self.VALID_REST_VERSION_87_51_MIN}))
 
     def verify_rest_version_for_pprc_cg(self):
         if '8.1' in self.backend['rest_version']:
@@ -239,14 +240,15 @@ class DS8KCommonHelper(object):
                 message=_("REST for DS8K 8.1 does not support PPRC "
                           "consistency group, please upgrade the CCL."))
         valid_rest_version = None
-        if ('5.7' in self.backend['rest_version'] and
+        rest_ver = self.backend['rest_version'][0:2]
+        if ('87' == rest_ver and
            dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.REST_VERSION_5_7_MIN_PPRC_CG)):
-            valid_rest_version = self.REST_VERSION_5_7_MIN_PPRC_CG
-        elif ('5.8' in self.backend['rest_version'] and
+           dist_version.LooseVersion(self.REST_VERSION_87_51_MIN_PPRC_CG)):
+            valid_rest_version = self.REST_VERSION_87_51_MIN_PPRC_CG
+        elif ('88' == rest_ver and
               dist_version.LooseVersion(self.backend['rest_version']) <
-              dist_version.LooseVersion(self.REST_VERSION_5_8_MIN_PPRC_CG)):
-            valid_rest_version = self.REST_VERSION_5_8_MIN_PPRC_CG
+              dist_version.LooseVersion(self.REST_VERSION_88_20_MIN_PPRC_CG)):
+            valid_rest_version = self.REST_VERSION_88_20_MIN_PPRC_CG
 
         if valid_rest_version:
             raise exception.VolumeDriverException(
@@ -277,10 +279,19 @@ class DS8KCommonHelper(object):
         else:
             pools_str = self.backend['pools_str'].replace(
                 ' ', '').upper().split(',')
-        pools = self._get_pools(pools_str)
-        unsorted_pools = self._format_pools(pools)
-        storage_pools = collections.OrderedDict(sorted(
-            unsorted_pools, key=lambda i: i[1]['capavail'], reverse=True))
+        pools = []
+        storage_pools = collections.OrderedDict()
+        for pid in pools_str:
+            try:
+                pools.append(self._get_pool(pid))
+            except restclient.APIException as e:
+                LOG.warning("Failed to get pool %(id)s information, "
+                            "Exception: %(ex)s.", {'id': pid,
+                                                   'ex': six.text_type(e)})
+        if len(pools):
+            unsorted_pools = self._format_pools(pools)
+            storage_pools = collections.OrderedDict(sorted(
+                unsorted_pools, key=lambda i: i[1]['capavail'], reverse=True))
         return storage_pools
 
     @proxy.logger
@@ -576,7 +587,7 @@ class DS8KCommonHelper(object):
 
     def _get_host(self, connector):
         # DS8K doesn't support hostname which is longer than 32 chars.
-        hname = ('OShost:%s' % filter_alnum(connector['host']))[:32]
+        hname = filter_alnum(connector['host'])[:32]
         os_type = connector.get('os_type')
         platform = connector.get('platform')
 
@@ -586,12 +597,39 @@ class DS8KCommonHelper(object):
             htype = 'iSeries'
         elif os_type == 'AIX':
             htype = 'pSeries'
-        elif platform in ('s390', 's390x') and os_type == 'linux2':
+        elif platform in ('s390', 's390x') and os_type in ('linux', 'linux2'):
             htype = 'zLinux'
         else:
             htype = 'LinuxRHEL'
         return collections.namedtuple('Host', ('name', 'type'))(hname, htype)
 
+    def check_vol_mapped_to_host(self, connector, vol_id):
+        map_info = {
+            'host_ports': [],
+            'mappings': [],
+            'lun_ids': []
+        }
+        host_wwpn_set = set(wwpn.upper() for wwpn in connector['wwpns'])
+        host_ports = self._get_host_ports(host_wwpn_set)
+        defined_hosts = set(
+            hp['host_id'] for hp in host_ports if hp['host_id'])
+        if not defined_hosts:
+            return False, None, map_info
+        elif len(defined_hosts) > 1:
+            raise restclient.APIException(_('More than one host found.'))
+        else:
+            host_id = defined_hosts.pop()
+            mappings = self._get_mappings(host_id)
+            lun_ids = [
+                m['lunid'] for m in mappings if m['volume']['id'] == vol_id]
+            map_info['host_ports'] = host_ports
+            map_info['mappings'] = mappings
+            map_info['lun_ids'] = lun_ids
+            if not lun_ids:
+                return False, host_id, map_info
+            else:
+                return True, host_id, map_info
+
     @coordination.synchronized('ibm-ds8k-{connector[host]}')
     def initialize_connection(self, vol_id, connector, **kwargs):
         host = self._get_host(connector)
@@ -635,58 +673,44 @@ class DS8KCommonHelper(object):
                 'target_lun': int(lun_id, 16),
                 'target_wwn': target_ports,
                 'initiator_target_map': {initiator: target_ports for
-                                         initiator in host_wwpn_set}
+                                         initiator in connector['wwpns']}
             }
         }
 
     @coordination.synchronized('ibm-ds8k-{connector[host]}')
-    def terminate_connection(self, vol_id, connector, force, **kwargs):
+    def terminate_connection(self, vol_id, host_id, connector, map_info):
         host = self._get_host(connector)
-        host_wwpn_set = set(wwpn.upper() for wwpn in connector['wwpns'])
-        host_ports = self._get_host_ports(host_wwpn_set)
-        defined_hosts = set(
-            hp['host_id'] for hp in host_ports if hp['host_id'])
+        host_ports = map_info['host_ports']
+        lun_ids = map_info['lun_ids']
+        mappings = map_info['mappings']
         delete_ports = set(
             hp['wwpn'] for hp in host_ports if not hp['host_id'])
         LOG.debug("terminate_connection: host_ports: %(host)s, "
                   "defined_hosts: %(defined)s, delete_ports: %(delete)s.",
                   {"host": host_ports,
-                   "defined": defined_hosts,
+                   "defined": host_id,
                    "delete": delete_ports})
-
-        if not defined_hosts:
-            LOG.info('Could not find host.')
-            return None
-        elif len(defined_hosts) > 1:
-            raise restclient.APIException(_('More than one host found.'))
-        else:
-            host_id = defined_hosts.pop()
-            mappings = self._get_mappings(host_id)
-            lun_ids = [
-                m['lunid'] for m in mappings if m['volume']['id'] == vol_id]
-            LOG.info('Volumes attached to host %(host)s are %(vols)s.',
-                     {'host': host_id, 'vols': ','.join(lun_ids)})
-            for lun_id in lun_ids:
-                self._delete_mappings(host_id, lun_id)
-            if not lun_ids:
-                LOG.warning("Volume %(vol)s is already not mapped to "
-                            "host %(host)s.",
-                            {'vol': vol_id, 'host': host.name})
-            # if this host only has volumes that have been detached,
-            # remove the host and its ports
-            ret_info = {
-                'driver_volume_type': 'fibre_channel',
-                'data': {}
-            }
-            if len(mappings) == len(lun_ids):
-                for port in delete_ports:
-                    self._delete_host_ports(port)
-                self._delete_host(host_id)
-                target_ports = [p['wwpn'] for p in self._get_ioports()]
-                target_map = {initiator.upper(): target_ports
-                              for initiator in connector['wwpns']}
-                ret_info['data']['initiator_target_map'] = target_map
-            return ret_info
+        for lun_id in lun_ids:
+            self._delete_mappings(host_id, lun_id)
+        if not lun_ids:
+            LOG.warning("Volume %(vol)s is already not mapped to "
+                        "host %(host)s.",
+                        {'vol': vol_id, 'host': host.name})
+        # if this host only has volumes that have been detached,
+        # remove the host and its ports
+        ret_info = {
+            'driver_volume_type': 'fibre_channel',
+            'data': {}
+        }
+        if len(mappings) == len(lun_ids):
+            for port in delete_ports:
+                self._delete_host_ports(port)
+            self._delete_host(host_id)
+            target_ports = [p['wwpn'] for p in self._get_ioports()]
+            target_map = {initiator: target_ports
+                          for initiator in connector['wwpns']}
+            ret_info['data']['initiator_target_map'] = target_map
+        return ret_info
 
     def create_group(self, group):
         return {'status': fields.GroupStatus.AVAILABLE}
@@ -738,16 +762,16 @@ class DS8KCommonHelper(object):
         self._client.send('DELETE', '/volumes/%s' % lun_id)
 
     def _get_version(self):
-        return self._client.fetchone('GET', '')
+        return self._client.fetchone('GET', '/systems')
 
     @proxy.logger
     def _create_lun(self, volData):
         return self._client.fetchid('POST', '/volumes', volData)
 
-    def _get_pools(self, pools_str):
-        return [self._client.fetchone('GET', '/pools/%s' % pid,
-                fields=['id', 'name', 'node', 'stgtype', 'cap', 'capavail'])
-                for pid in pools_str]
+    def _get_pool(self, pool_id):
+        return self._client.fetchone('GET', '/pools/%s' % pool_id,
+                                     fields=['id', 'name', 'node', 'stgtype',
+                                             'cap', 'capavail'])
 
     def start_flashcopy(self, vol_pairs, freeze=False):
         options = [
@@ -1020,8 +1044,8 @@ class DS8KECKDHelper(DS8KCommonHelper):
     OPTIONAL_PARAMS = ['ds8k_host_type', 'port_pairs', 'ds8k_ssid_prefix',
                        'lss_range_for_cg']
     # if use new REST API, please update the version below
-    VALID_REST_VERSION_5_7_MIN = '5.7.51.1068'
-    VALID_REST_VERSION_5_8_MIN = '5.8.20.1059'
+    VALID_REST_VERSION_87_51_MIN = '87.51.63.0'
+    VALID_REST_VERSION_88_20_MIN = '88.20.112.0'
     MIN_VALID_STORAGE_VERSION = '8.1'
     INVALID_STORAGE_VERSION = '8.0.1'
 
@@ -1072,19 +1096,20 @@ class DS8KECKDHelper(DS8KCommonHelper):
            dist_version.LooseVersion(self.MIN_VALID_STORAGE_VERSION)):
             self._disable_thin_provision = True
 
-        if (('5.7' in self.backend['rest_version'] and
+        rest_ver = self.backend['rest_version'][0:2]
+        if (('87' == rest_ver and
            dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.VALID_REST_VERSION_5_7_MIN)) or
-           ('5.8' in self.backend['rest_version'] and
+           dist_version.LooseVersion(self.VALID_REST_VERSION_87_51_MIN)) or
+           ('88' == rest_ver and
            dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.VALID_REST_VERSION_5_8_MIN))):
+           dist_version.LooseVersion(self.VALID_REST_VERSION_88_20_MIN))):
             raise exception.VolumeDriverException(
                 message=(_("REST version %(invalid)s is lower than "
                            "%(valid)s, please upgrade it in DS8K.")
                          % {'invalid': self.backend['rest_version'],
-                            'valid': (self.VALID_REST_VERSION_5_7_MIN if '5.7'
-                                      in self.backend['rest_version'] else
-                                      self.VALID_REST_VERSION_5_8_MIN)}))
+                            'valid': (self.VALID_REST_VERSION_87_51_MIN
+                                      if '87' == rest_ver else
+                                      self.VALID_REST_VERSION_88_20_MIN)}))
 
     @proxy.logger
     def _get_device_mapping(self):
