@@ -13,8 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-"""
-This is the driver that allows openstack to talk to DS8K.
+"""This is the driver that allows openstack to talk to DS8K.
 
 All volumes are thin provisioned by default, if the machine is licensed for it.
 This can be overridden by creating a volume type and specifying a key like so:
@@ -62,11 +61,11 @@ Sample settings for cinder.conf:
 """
 import ast
 import json
-import six
 
 import eventlet
 from oslo_config import cfg
 from oslo_log import log as logging
+import six
 
 from cinder import context
 from cinder import coordination
@@ -144,9 +143,10 @@ class Lun(object):
         2.1.1 - Added support for replication consistency group.
         2.1.2 - Added support for cloning volume asynchronously.
         2.3.0 - Added support for reporting backend state.
+        2.5.0 - Added support for revert to snapshot operation.
     """
 
-    VERSION = "2.3.0"
+    VERSION = "2.5.0"
 
     class FakeLun(object):
 
@@ -223,7 +223,7 @@ class Lun(object):
             self.ds_id = provider_location['vol_hex_id']
         else:
             self.ds_id = None
-        self.cinder_name = volume.display_name
+        self.cinder_name = volume.name
         self.pool_lss_pair = {}
         self.is_snapshot = is_snapshot
         if self.is_snapshot:
@@ -231,20 +231,14 @@ class Lun(object):
                           if volume.group_snapshot else None)
             self.size = volume.volume_size
             # ds8k supports at most 16 chars
-            self.ds_name = (
-                "OS%s:%s" % ('snap', helper.filter_alnum(self.cinder_name))
-            )[:16]
+            self.ds_name = helper.filter_alnum(self.cinder_name)[:16]
             self.metadata = self._get_snapshot_metadata(volume)
             self.source_volid = volume.volume_id
         else:
             self.group = Group(volume.group) if volume.group else None
             self.size = volume.size
-            self.ds_name = (
-                "OS%s:%s" % ('vol', helper.filter_alnum(self.cinder_name))
-            )[:16]
-            self.replica_ds_name = (
-                "OS%s:%s" % ('Replica', helper.filter_alnum(self.cinder_name))
-            )[:16]
+            self.ds_name = helper.filter_alnum(self.cinder_name)[:16]
+            self.replica_ds_name = helper.filter_alnum(self.cinder_name)[:16]
             self.previous_status = volume.previous_status
             self.replication_status = volume.replication_status
             self.replication_driver_data = (
@@ -1066,6 +1060,31 @@ class DS8KProxy(proxy.IBMStorageProxy):
             volume_update = lun.get_volume_update()
         return True, volume_update
 
+    @proxy._trace_time
+    @proxy.logger
+    def revert_to_snapshot(self, context, volume, snapshot):
+        """Revert volume to snapshot."""
+        if snapshot.volume_size != volume.size:
+            raise exception.InvalidInput(
+                reason=_('Reverting volume is not supported if the volume '
+                         'size is not equal to the snapshot size.'))
+
+        vol_lun = Lun(volume)
+        snap_lun = Lun(snapshot, is_snapshot=True)
+        if vol_lun.type_replication:
+            raise exception.VolumeDriverException(
+                message=_('Driver does not support revert to snapshot '
+                          'of replicated volume.'))
+
+        try:
+            self._clone_lun(snap_lun, vol_lun)
+        except Exception as err:
+            msg = (_("Reverting volume %(vol)s to snapshot %(snap)s failed "
+                     "due to: %(err)s.")
+                   % {"vol": volume.name, "snap": snapshot.name, "err": err})
+            LOG.error(msg)
+            raise exception.VolumeBackendAPIException(data=msg)
+
     @proxy._trace_time
     @proxy.logger
     def initialize_connection(self, volume, connector, **kwargs):
