@@ -14,8 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 #
-"""
-ISCSI volume driver for IBM Storwize family and SVC storage systems.
+"""ISCSI volume driver for IBM Storwize family and SVC storage systems.
 
 Notes:
 1. If you specify both a password and a key file, this driver will use the
@@ -45,7 +44,6 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import interface
 from cinder.volume import configuration as conf
-
 from cinder.volume.drivers.ibm.storwize_svc import (
     storwize_svc_common as storwize_common)
 
@@ -96,9 +94,10 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         2.2.2 - Add replication group support
         2.2.3 - Add backup snapshots support
         2.2.4 - Add hyperswap support
+        2.2.5 - Add support for host attachment using portsets
     """
 
-    VERSION = "2.2.4"
+    VERSION = "2.2.5"
 
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "IBM_STORAGE_CI"
@@ -109,6 +108,10 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         self.configuration.append_config_values(
             storwize_svc_iscsi_opts)
 
+    @staticmethod
+    def get_driver_options():
+        return storwize_common.storwize_svc_opts + storwize_svc_iscsi_opts
+
     def validate_connector(self, connector):
         """Check connector for at least one enabled iSCSI protocol."""
         if 'initiator' not in connector:
@@ -164,24 +167,41 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
             volume_name, backend_helper, node_state = self._get_vol_sys_info(
                 volume)
 
+        backend_helper.initialize_host_info()
+
         host_site = self._get_volume_host_site_from_conf(volume,
                                                          connector,
                                                          iscsi=True)
-        is_hyper_volume = backend_helper.is_volume_hyperswap(volume_name)
+        is_hyper_volume = self.is_volume_hyperswap(volume)
         if is_hyper_volume and host_site is None:
             msg = (_('There is no correct storwize_preferred_host_site '
                      'configured for a hyperswap volume %s.') % volume.name)
             LOG.error(msg)
             raise exception.VolumeDriverException(message=msg)
 
-        # Check if a host object is defined for this host name
-        host_name = backend_helper.get_host_from_connector(connector,
-                                                           iscsi=True)
-        if host_name is None:
-            # Host does not exist - add a new host to Storwize/SVC
-            host_name = backend_helper.create_host(connector, iscsi=True,
-                                                   site=host_site)
-        elif is_hyper_volume:
+        # Try creating the host, if host creation is successfull continue
+        # with intialization flow, else search for host object defined for
+        # this connector info.
+        host_name = None
+        try:
+            opts = self._get_vdisk_params(volume.volume_type_id)
+            host_name = (
+                backend_helper.create_host(connector, iscsi=True,
+                                           site=host_site,
+                                           portset=opts['storwize_portset']))
+        except exception.VolumeBackendAPIException as excp:
+            if "CMMVC6578E" in excp.msg:
+                msg = (_('Host already exists for connector '
+                         '%(conn)s'), {'conn': connector})
+                LOG.info(msg)
+                host_name = backend_helper.get_host_from_connector(connector,
+                                                                   iscsi=True)
+            else:
+                msg = (_('Error creating host %(ex)s'), {'ex': excp.msg})
+                LOG.error(msg)
+                raise exception.VolumeDriverException(message=msg)
+
+        if is_hyper_volume:
             self._update_host_site_for_hyperswap_volume(host_name, host_site)
 
         chap_secret = backend_helper.get_chap_secret_for_host(host_name)
@@ -197,13 +217,15 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
 
         try:
             properties = self._get_single_iscsi_data(volume, connector,
-                                                     lun_id, chap_secret)
+                                                     lun_id, chap_secret,
+                                                     opts['storwize_portset'])
             multipath = connector.get('multipath', False)
             if multipath:
-                properties = self._get_multi_iscsi_data(volume, connector,
-                                                        lun_id, properties,
-                                                        backend_helper,
-                                                        node_state)
+                properties = (
+                    self._get_multi_iscsi_data(volume, connector,
+                                               lun_id, properties,
+                                               backend_helper, node_state,
+                                               opts['storwize_portset']))
         except Exception as ex:
             with excutils.save_and_reraise_exception():
                 LOG.error('initialize_connection: Failed to export volume '
@@ -224,7 +246,8 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
 
         return {'driver_volume_type': 'iscsi', 'data': properties, }
 
-    def _get_single_iscsi_data(self, volume, connector, lun_id, chap_secret):
+    def _get_single_iscsi_data(self, volume, connector, lun_id,
+                               chap_secret, portset):
         LOG.debug('enter: _get_single_iscsi_data: volume %(vol)s with '
                   'connector %(conn)s lun_id %(lun_id)s',
                   {'vol': volume.id, 'conn': connector,
@@ -261,6 +284,11 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         # Get preferred node and other nodes in I/O group
         preferred_node_entry = None
         io_group_nodes = []
+        if node_state['code_level'] >= (8, 4, 2, 0):
+            backend_helper.add_iscsi_ip_addrs(node_state['storage_nodes'],
+                                              node_state['code_level'],
+                                              portset=portset)
+
         for node in node_state['storage_nodes'].values():
             if self.protocol not in node['enabled_protocols']:
                 continue
@@ -289,10 +317,14 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
             'target_lun': lun_id,
             'volume_id': volume.id}
 
-        if preferred_node_entry['ipv4']:
-            ipaddr = preferred_node_entry['ipv4'][0]
+        if node_state['code_level'] >= (8, 4, 2, 0):
+            if preferred_node_entry['IP_address']:
+                ipaddr = preferred_node_entry['IP_address'][0]
         else:
-            ipaddr = preferred_node_entry['ipv6'][0]
+            if preferred_node_entry['ipv4']:
+                ipaddr = preferred_node_entry['ipv4'][0]
+            else:
+                ipaddr = preferred_node_entry['ipv6'][0]
         properties['target_portal'] = '%s:%s' % (ipaddr, '3260')
         properties['target_iqn'] = preferred_node_entry['iscsi_name']
         if chap_secret:
@@ -311,14 +343,18 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         return properties
 
     def _get_multi_iscsi_data(self, volume, connector, lun_id, properties,
-                              backend_helper, node_state):
+                              backend_helper, node_state, portset):
         LOG.debug('enter: _get_multi_iscsi_data: volume %(vol)s with '
                   'connector %(conn)s lun_id %(lun_id)s',
                   {'vol': volume.id, 'conn': connector,
                    'lun_id': lun_id})
 
         try:
-            resp = backend_helper.ssh.lsportip()
+            if node_state['code_level'] >= (8, 4, 2, 0):
+                portset_name = portset if portset else 'portset0'
+                resp = backend_helper.ssh.lsip(portset=portset_name)
+            else:
+                resp = backend_helper.ssh.lsportip()
         except Exception as ex:
             msg = (_('_get_multi_iscsi_data: Failed to '
                      'get port ip because of exception: '
@@ -335,11 +371,14 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
                     continue
                 link_state = ip_data.get('link_state', None)
                 valid_port = ''
-                if ((ip_data['state'] == 'configured' and
-                        link_state == 'active') or
-                        ip_data['state'] == 'online'):
-                    valid_port = (ip_data['IP_address'] or
-                                  ip_data['IP_address_6'])
+                if node_state['code_level'] >= (8, 4, 2, 0):
+                    valid_port = ip_data['IP_address']
+                else:
+                    if ((ip_data['state'] == 'configured' and
+                            link_state == 'active') or
+                            ip_data['state'] == 'online'):
+                        valid_port = (ip_data['IP_address'] or
+                                      ip_data['IP_address_6'])
                 if valid_port:
                     properties['target_portals'].append(
                         '%s:%s' % (valid_port, '3260'))
@@ -379,6 +418,23 @@ class StorwizeSVCISCSIDriver(storwize_common.StorwizeSVCCommonDriver):
         # In this case construct the lock without the host property
         # so that all the fake connectors to an SVC are serialized
         host = connector['host'] if 'host' in connector else ""
+        attachment_count = 0
+        if hasattr(volume, 'multiattach') and volume.multiattach:
+            try:
+                attachment_list = volume.volume_attachment
+                for attachment in attachment_list:
+                    if (attachment.attach_status == "attached" and
+                       attachment.attached_host == host):
+                        attachment_count += 1
+            except AttributeError:
+                pass
+            if attachment_count > 1:
+                LOG.debug("Volume %(volume)s is attached to multiple "
+                          "instances on host %(host_name)s, "
+                          "skip terminate volume connection",
+                          {'volume': volume.name,
+                           'host_name': volume.host.split('@')[0]})
+                return
 
         @coordination.synchronized('storwize-host-{system_id}-{host}')
         def _do_terminate_connection_locked(system_id, host):
