@@ -19,13 +19,27 @@ Volume driver for NetApp Data ONTAP FibreChannel storage systems.
 from cinder import interface
 from cinder.volume import driver
 from cinder.volume.drivers.netapp.dataontap import block_cmode
+from cinder.volume.drivers.netapp import options as na_opts
 from cinder.zonemanager import utils as fczm_utils
 
 
 @interface.volumedriver
 class NetAppCmodeFibreChannelDriver(driver.BaseVD,
                                     driver.ManageableVD):
-    """NetApp C-mode FibreChannel volume driver."""
+    """NetApp C-mode FibreChannel volume driver.
+
+    Version history:
+
+    .. code-block:: none
+
+        1.0.0 - Driver development before Wallaby
+        2.0.0 - Wallaby driver version bump
+        3.0.0 - Add support for Intra-cluster Storage assisted volume migration
+                Add support for revert to snapshot
+
+    """
+
+    VERSION = "3.0.0"
 
     DRIVER_NAME = 'NetApp_FibreChannel_Cluster_direct'
 
@@ -38,6 +52,10 @@ class NetAppCmodeFibreChannelDriver(driver.BaseVD,
         self.library = block_cmode.NetAppBlockStorageCmodeLibrary(
             self.DRIVER_NAME, 'FC', **kwargs)
 
+    @staticmethod
+    def get_driver_options():
+        return na_opts.netapp_cluster_opts
+
     def do_setup(self, context):
         self.library.do_setup(context)
 
@@ -94,14 +112,16 @@ class NetAppCmodeFibreChannelDriver(driver.BaseVD,
     def unmanage(self, volume):
         return self.library.unmanage(volume)
 
-    @fczm_utils.add_fc_zone
     def initialize_connection(self, volume, connector):
-        return self.library.initialize_connection_fc(volume, connector)
+        conn_info = self.library.initialize_connection_fc(volume, connector)
+        fczm_utils.add_fc_zone(conn_info)
+        return conn_info
 
-    @fczm_utils.remove_fc_zone
     def terminate_connection(self, volume, connector, **kwargs):
-        return self.library.terminate_connection_fc(volume, connector,
-                                                    **kwargs)
+        conn_info = self.library.terminate_connection_fc(volume, connector,
+                                                         **kwargs)
+        fczm_utils.remove_fc_zone(conn_info)
+        return conn_info
 
     def get_pool(self, volume):
         return self.library.get_pool(volume)
@@ -133,3 +153,9 @@ class NetAppCmodeFibreChannelDriver(driver.BaseVD,
     def failover_host(self, context, volumes, secondary_id=None, groups=None):
         return self.library.failover_host(
             context, volumes, secondary_id=secondary_id)
+
+    def migrate_volume(self, context, volume, host):
+        return self.library.migrate_volume(context, volume, host)
+
+    def revert_to_snapshot(self, context, volume, snapshot):
+        return self.library.revert_to_snapshot(volume, snapshot)
