@@ -357,16 +357,6 @@ class NexentaISCSIDriver(driver.ISCSIDriver):
                            'tg': mapping_tg, 'hg': mapping_hg})
         return info
 
-    def get_volume_stats(self, refresh=False):
-        """Get volume stats.
-
-        If 'refresh' is True, run update the stats first.
-        """
-        if refresh or not self._stats:
-            self._update_volume_stats()
-
-        return self._stats
-
     def _update_volume_stats(self):
         """Retrieve stats info for NexentaStor appliance."""
         LOG.debug('Updating volume backend %(volume_backend_name)s stats',
