@@ -304,7 +304,6 @@ class RemoteFSDriver(driver.BaseVD):
 
         if encrypted:
             encryption = volume_utils.check_encryption_provider(
-                self.db,
                 volume,
                 volume.obj_context)
 
@@ -1393,7 +1392,7 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                   {'vol': volume.id, 'snap': snapshot.id})
 
         status = snapshot.status
-        acceptable_states = ['available', 'backing-up', 'restoring']
+        acceptable_states = ['available', 'backing-up']
         self._validate_state(status, acceptable_states,
                              obj_description='snapshot',
                              invalid_exc=exception.InvalidSnapshot)
@@ -1480,7 +1479,6 @@ class RemoteFSSnapDriverBase(RemoteFSDriver):
                 file_json = jsonutils.dumps(file_json_dict)
 
                 encryption = volume_utils.check_encryption_provider(
-                    db=db,
                     volume=snapshot.volume,
                     context=obj_context)
 
