@@ -15,8 +15,6 @@
 
 """StorPool block device driver"""
 
-from __future__ import absolute_import
-
 import platform
 
 from oslo_config import cfg
@@ -28,6 +26,7 @@ import six
 from cinder import exception
 from cinder.i18n import _
 from cinder import interface
+from cinder.volume import configuration
 from cinder.volume import driver
 from cinder.volume import volume_types
 
@@ -54,7 +53,7 @@ storpool_opts = [
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(storpool_opts)
+CONF.register_opts(storpool_opts, group=configuration.SHARED_CONF_GROUP)
 
 
 class StorPoolConfigurationInvalid(exception.CinderException):
@@ -94,7 +93,7 @@ class StorPoolDriver(driver.VolumeDriver):
     """
 
     VERSION = '1.2.3'
-    CI_WIKI_NAME = 'StorPool_CI'
+    CI_WIKI_NAME = 'StorPool_distributed_storage_CI'
 
     def __init__(self, *args, **kwargs):
         super(StorPoolDriver, self).__init__(*args, **kwargs)
@@ -263,12 +262,6 @@ class StorPoolDriver(driver.VolumeDriver):
             LOG.error("StorPoolDriver API initialization failed: %s", e)
             raise
 
-    def get_volume_stats(self, refresh=False):
-        if refresh:
-            self._update_volume_stats()
-
-        return self._stats
-
     def _update_volume_stats(self):
         try:
             dl = self._attach.api().disksList()
